<?php

/**
 * Handler to migrate communities from Epractice website
 * @author antartem
 *
 */
class EpracticeCommunityMigration extends Migration {
	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {
		//
		parent::__construct(MigrateGroup::getInstance('isa_epractice_migration'));

		$json_file =  MIGRATION_FILES_JSON.'/community.json';
		$this->map = new MigrateSQLMap($this->machineName,
			array(
				'nid' => array(
					'type' => 'int',
					'unsigned' => TRUE,
					'not null' => TRUE,
				),
			),
			MigrateDestinationNode::getKeySchema()
		);

		$this->dependencies[] = 'EpracticeTaxonomy';
		$this->dependencies[] = 'EpracticeUser';

		$this->source = new MigrateSourceJSON($json_file , 'nid', $this->fields());

		$this->destination = new MigrateDestinationNode('community');

		$this->addFieldMapping('uid', 'uid')->defaultValue(0)
			->sourceMigration('EpracticeUser');
		$this->addSimpleMappings(array('title', 'format', 'created','status','language'));
		$this->addFieldMapping('body', 'og_description');
		$this->addFieldMapping('og_description', 'og_description')
			->callbacks(array($this, 'getAbstract'));
		$this->addFieldMapping('field_community_short_name', 'field_community_url')
		->callbacks(array($this, 'getStringFromArray'));
		$this->addFieldMapping('field_community_expectations', 'field_community_objectives');
		$this->addFieldMapping('field_community_estimated_launch', 'field_community_est_launch');
		$this->addFieldMapping('field_community_estimated_after', 'field_community_est_year');
		$this->addFieldMapping('field_community_email', 'field_community_email');
		$this->addFieldMapping('taxonomy', 'taxonomy')
			->sourceMigration('EpracticeTaxonomy')
			->arguments(array('source_type' => 'tid'));
	}


	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
				'nid' => 'Node id',
				'uid' => 'User id',
				'language' => '',
				'format' => '',
				'created' => '',
				'changed' => '',
				'status' => '',
				'title' => '',
				'body' => '',
				'teaser' => '',
				'og_description' => '',
				'field_community_url' => '',
				'field_community_existing_url' => '',
				'field_community_logo' => '',
				'field_community_sponsor' => '',
				'field_community_domain' => '',
				'field_community_topic' => '',
				'field_community_tags' => '',
				'field_community_location' => '',
				'field_community_language' => '',
				'field_community_email' => '',
				'field_community_objectives' => '',
				'field_community_est_launch' => '',
				'field_community_est_year' => '',
				'field_community_existing' => '',
				'field_community_content' => '',
				'field_community_content_cases' => '',
				'field_community_content_news' => '',
				'field_community_content_library' => '',
				'field_community_content_ew' => '',
				'field_community_criteria' => '',
				'field_community_terms' => '',
				'field_community_position' => '',
		);
	}

	/**
	 * Change the format of input taxonomy data
	 * @see Migration::prepareRow()
	 * @param $row
	 * @return bool Always import data
	 */
	public function prepareRow($row){
		_cast_taxonomy_to_array($row);
		_import_topic_taxonomy($row);
 		_update_textarea_links($row);

		return TRUE;
	}

	/**
	 * Change multiple array fields and rise it one level
	 * Import related files
	 * @param stdClass $node
	 * @param stdClass $row
	 */
	public function prepare($node, stdClass $row) {
		_cast_node_object_to_array($node);
		_epractice_file_import($row->field_community_logo, $node->field_community_logo, $node->uid);
		_epractice_file_import($row->field_community_sponsor, $node->field_community_sponsor_logo, $node->uid);
		_add_urls($row->field_community_existing[0], $node->field_community_url);
		//_epractice_og_content($node, $row->field_community_domain);
		if($node->taxonomy){
		  $node->taxonomy = array_unique($node->taxonomy);
		}
		foreach($row->field_community_content as $content_option){
		  if($content_option->value=='news'){
		    $node->field_community_news_creation[0]['value'] = 'Activated';
		  }
		  if($content_option->value=='library'){
		    $node->field_community_documents_creati[0]['value'] = 'Activated';
		  }
		}
		$node->field_community_forum_creation[0]['value'] = 'Deactivated';
		$node->field_community_wiki_creation[0]['value'] = 'Deactivated';
	}

    /**
     * Import current workflow state
     * @param $node
     * @param stdClass $row
     */
    public function complete($node, stdClass $row){
      if($sid = _get_workflow_state($row->_workflow, $node->type)){
        workflow_execute_transition($node, $sid, 'Automatic Epractice Migration Process', TRUE);
      }
	  _highlight_content($node, $row->field_community_domain);
	  //Highlight communities on the epractice community
	  _highlight_content($node, variable_get('community_epractice_epractice',0));
	  _update_changed_date($node->nid,$row->changed);
    }

	/**
	 * Generates abstract text
	 * @param string $value
	 * @return string
	 */
	function getAbstract($value){
	  $abstract = substr($value,0,200);
	  $abstract .="...";

	  return $abstract;
	}

	/**
	 * Generate string from an array
	 * @param array $value
	 * @return string
	 */
	function getStringFromArray($value){
	  if(is_array($value)){
		return strtolower($value[0]->value);
	  }

	  return strtolower($value);
	}
}