<?php

define('FILTER_FORMAT_FILTERED', 1);
define('FILTER_FORMAT_FULL', 2);
define('FILTER_FORMAT_PHP', 3);
define('FILTER_FORMAT_POPUP', 5);

/**
 * Add pseudo-field date-of-creation based on node created time.
 * The pseudo-field is inserted before the "date of last modification" field
 * The pseudo-field is not inserted if the "date of last modification" field is not found
 *
 * @param $vars
 *   An array of variables to pass to the theme template.
 *
 * @see node.tpl.php
 * @see template.php
 */
function _joinup_add_field_date_created(&$vars) {
  // https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-744
  // insert a new field with the date of creation, before field_asset_last_modification, so that they will appear in that order in the page
  $vars['fields'] = array_values($vars['fields']);  // to force indexing based on 0 (0,1,2...)
  $_date_mod_pos = array_search('field_asset_last_modification',$vars['fields']);
  if (false!==$_date_mod_pos) {
    array_splice($vars['fields'], $_date_mod_pos, 0, array('date_created'));
    // define the data for the new field
    $vars['date_created'] = array(
      'label' => 'Date of creation',  // the template will wrap the string with the t() function
      'value' => date(variable_get('date_format_joinup_date_format','j M Y - H:i'),$vars['node']->created)    // not using drupal's format_date to avoid time-zone adjustments
    );
  }
}


/**
 * Override or insert variables into the "node.tpl.php" templates.
 *
 * @param $vars
 *   An array of variables to pass to the theme template.
 *
 * @see node.tpl.php
 * @see template.php
 */
function _joinup_preprocess_node(&$vars) {
  $function_name = '_joinup_preprocess_' . $vars['node']->type;
  if (function_exists($function_name)) {
     $function_name($vars);
  }
  // don't display $submitted, flag and rating if he's wiki licence wizard
  if ($vars['node']->type == ISA_WIKI_TYPE && isa_toolbox_is_license_wizard_wiki($vars['node'])) {
    unset($vars['submitted']);
  } else {

    // Prepare flag to display
    // machine name for flags you want displayed
    $flags_to_display = array('editor_choice', 'call_for_review');
    $flags_counts = flag_get_counts('node', $vars['node']->nid);
    if ($flags_counts) {
      $flags = '';
      foreach ($flags_counts as $flag_name => $count) {
        if (in_array($flag_name, $flags_to_display)) {
          $flag = flag_get_flag($flag_name);
          $flag_class = joinup_clean_css_identifier(strtolower($flag->title));
          $flag_title = check_plain($flag->title);
          $flags .= '<div class="' . $flag_class . '" title="' . $flag_title . '">' . $flag_title . '</div>';
        }
      }
    }
    $vars['flags_view'] = !empty($flags) ? $flags : NULL;
    $statistics_counter = $vars['node']->links['statistics_counter']['title'];
    // Prepare vote rating display
    if (variable_get('fivestar_title_' . $vars['node']->type, -1) != -1) {
      $vars['vote_rating'] = joinup_prepare_vote_rating($vars['node']);
      if ($statistics_counter) {
        $vars['vote_rating'] .= ' | ' . $statistics_counter;
      }
    } else {
      $vars['vote_rating'] = $statistics_counter;
    }
    //Prepare a title linked to the node
    $vars['title_link'] = l($vars['title'], $vars['path']);
  }
  $vars['date_creation'] = date("d M Y - H:i", $vars['node']->created);
  $vars['date_modification'] = date("d M Y - H:i", $vars['node']->changed);
}

/**
 * Override or insert variables into the "node-publisher.tpl.php" template.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see node-publisher.tpl.php
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_publisher(&$vars) {
  $node = $vars['node'];

  $exclude = array(
    'field_publisher_description',
    'field_publisher_name'
  );

  $language_textfields = array(array('field_name', 'field_publisher_name', 'title'));
  _joinup_prepare_template($language_textfields, $vars, $exclude);

  //Prepare taxonomies
  $include = array(
    variable_get("publisher_type_vid", 72),
  );
  isa_toolbox_create_taxonomy_list($node, $include, FALSE, TRUE);
  foreach ($node->taxonomy_terms as $vocab => $terms) {
    $voc = str_replace(' ', '', strtolower($vocab));
    $vars['taxonomy_terms_' . $voc]['label'] = $vocab;
    $vars['taxonomy_terms_' . $voc]['value'] = $terms;
    $vars['fields'][] = 'taxonomy_terms_' . $voc;
  }
}

/**
 * Override or insert variables into the "node-licence.tpl.php" template.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see node-licence.tpl.php
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_licence(&$vars) {
  $node = $vars ['node'];

  $exclude = array(
    'field_licence_description',
    'field_licence_name'
  );

  $language_textfields = array(array('description', 'field_licence_description', 'body'),
    array('field_name', 'field_licence_name', 'title'));
  _joinup_prepare_template($language_textfields, $vars, $exclude);

  //Prepare taxonomies
  $include = array(
    variable_get("licence_type_vid", 75),
  );
  isa_toolbox_create_taxonomy_list($node, $include, FALSE, TRUE);
  foreach ($node->taxonomy_terms as $vocab => $terms) {
    $voc = str_replace(' ', '', strtolower($vocab));
    $vars['taxonomy_terms_' . $voc]['label'] = $vocab;
    $vars['taxonomy_terms_' . $voc]['value'] = $terms;
    $vars['fields'][] = 'taxonomy_terms_' . $voc;
  }
}

/**
 * Override or insert variables into the "node-distribution.tpl.php" template.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see node-distribution.tpl.php
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_distribution(&$vars) {
  $node = $vars['node'];
  variable_set('filefield_icon_theme', 'joinup');
  $exclude = array(
    'field_distribution_description',
    'field_distribution_name'
  );

  //Display only the access url or the file
  if (isset($node->field_distribution_access_url1[0]['url'])) {
    $exclude[] = 'field_distribution_access_url';
  }
  else {
    $exclude[] = 'field_distribution_access_url1';
  }
  $language_textfields = array(array('description', 'field_distribution_description', 'body'),
    array('field_name', 'field_distribution_name', 'title'));
  _joinup_prepare_template($language_textfields, $vars, $exclude);

  //Prepare taxonomies
  $include = array(
    variable_get("file_format_vid", 73),
    variable_get("status_vid", 69),
    variable_get("representation_technique_vid", 70),
  );
  isa_toolbox_create_taxonomy_list($node, $include, FALSE, TRUE);
  foreach ($node->taxonomy_terms as $vocab => $terms) {
    $voc = str_replace(' ', '', strtolower($vocab));
    $vars['taxonomy_terms_' . $voc]['label'] = $vocab;
    $vars['taxonomy_terms_' . $voc]['value'] = $terms;
    $vars['fields'][] = 'taxonomy_terms_' . $voc;
  }
  // https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-744
  // insert a new field with the date of creation, before field_asset_last_modification, so that they will appear in that order in the page
  _joinup_add_field_date_created($vars);

}

/**
 * Override or insert variables into the "node-item.tpl.php" template.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see node-item.tpl.php
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_item(&$vars) {
  $node = $vars['node'];

  $exclude = array(
    'field_item_description',
    'field_item_label'
  );

  $language_textfields = array(array('description', 'field_item_description', 'body'),
    array('field_name', 'field_item_label', 'title'));
  _joinup_prepare_template($language_textfields, $vars, $exclude);
}

/**
 * Override or insert variables into the "node-contact_point.tpl.php" template.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see node-contact_point.tpl.php
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_contact_point(&$vars) {
  _joinup_prepare_template(array(), $vars, $exclude);
}

/**
 * Override or insert variables into the "node-documentation.tpl.php" template.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see node-contact_point.tpl.php
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_documentation(&$vars) {
  $node = $vars['node'];
//https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-653
//Only show the field with data
  $exluir = NULL;
  if (isset($node->field_documentation_access_url[0]['fid']) && $node->field_documentation_access_url[0]['fid']>0){
	$exluir = 'field_documentation_access_url1';
  } elseif (isset($node->field_documentation_access_url1[0]['url']) && $node->field_documentation_access_url1[0]['url'] != ''){
	$exluir = 'field_documentation_access_url';
  }
  $exclude = array('field_documentation_title', $exluir);
  $language_textfields = array(array('field_name', 'field_documentation_title', 'title'));
  _joinup_prepare_template($language_textfields, $vars, $exclude);
}

/**
 * Override or insert variables into the "node-asset_release.tpl.php" template.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see node-asset_release.tpl.php
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_asset_release(&$vars) {
  global $user;
  $node = $vars['node'];
  foreach($node->field_asset_temporal_coverage as $tc){
    $dateWrong1 = date('j F Y', strtotime($tc['value']));
    $dateRigth1 = date('j M Y', strtotime($tc['value']));
    $tc['view'] = str_replace($dateWrong1, $dateRigth1, $tc['view']);
    $dateWrong2 = date('j F Y', strtotime($tc['value2']));
    $dateRigth2 = date('j M Y', strtotime($tc['value2']));
    $tc['view'] = str_replace($dateWrong2, $dateRigth2, $tc['view']);
    $tcs[] = $tc;
  }
  $node->field_asset_temporal_coverage = $tcs;

  $exclude = array(
    'field_asset_description',
    'field_asset_distribution',
    'field_asset_name',
    'field_asset_node_reference'
  );

  $type = $node->field_asset_release_type[0]['value'];
  $release_fields_by_type = _asset_release_fields_by_type();
  if( $type == 'asset'){
    $exclude = array_merge( $exclude, $release_fields_by_type['software'] );
  } else {
    $exclude = array_merge( $exclude, $release_fields_by_type['asset'] );
  }
  $language_textfields = array(array('description', 'field_asset_description', 'body'),
    array('field_name', 'field_asset_name', 'title'));
  _joinup_prepare_template($language_textfields, $vars, $exclude);

  // Append the version number to the title
  if (!empty($node->field_asset_version[0]['value']['field_language_textfield_name'][0]['value'])) {
    $asset_version = check_plain($node->field_asset_version[0]['value']['field_language_textfield_name'][0]['value']);
    drupal_set_title("{$vars['title']}&nbsp;{$asset_version}");
  }

  // Prepare the distributions
  if (!empty($node->field_asset_distribution)) {
    foreach ($node->field_asset_distribution as $item) {
      $item = node_load($item['nid']);
      if($item){
        if (isset($item->field_distribution_access_url[0])) {
          // field_distribution_access_url ==> File
          if(!empty($item->field_distribution_access_url[0]['filepath'])){
            $name = explode('.',$item->field_distribution_access_url[0]['filename']);
            $name = end($name);
            $path = $item->field_distribution_access_url[0]['filepath'];
            if (isset($user->roles[1])) {
              $url = "anonymous_validation/{$node->nid}/{$item->field_distribution_access_url[0]['fid']}";
              $class = 'popups';
            }
            else {
              $url = file_create_url($path);
              // Force path to be outputted as system/files instead of sites/default/files (for statistics)
              $url = str_replace('/sites/default/files/', '/system/files/', $url);
            }

            $options = array(
              'attributes' => array(
                'type' => $item->field_distribution_access_url[0]['filemime'] . '; length=' . $item->field_distribution_access_url[0]['filesize'],
                'class' => isset($class) ? $class : NULL,
              ),
            );
          }
        } else if (isset($item->field_distribution_access_url1[0])) {
          $name = t('Url');
          if(!empty($item->field_distribution_access_url1[0]['url'])){
            $url = $item->field_distribution_access_url1[0]['url'];
            $options['attributes'] = array(
              'class' => 'asset-external-download nid-' . $node->nid . ' did-' . $item->nid
            );
          }
        }
        if($url){
          // Construction of the link to the file/url
          $dist['download'] = l(strtoupper($name), $url, $options);
        }else{
          $dist['download'] = t('Download not available');
        }
        // Construct of the link to the Distribution
        $dist['link'] = l($item->title, "node/{$item->nid}");

        if (count($dist) > 0) {
          $vars['distributions'][] = $dist;
        }
      }
    }
  }

  //Prepare repository of origin
  $groups = og_get_node_groups($node);
  foreach ($groups as $key => $value) {
    $group = node_load($key);
    if ($group->group_type == 'repository') {
      $vars['fields'][] = 'repository_origin';
      $vars['repository_origin'] = array(
        'label' => t('Repository of origin'),
        'value' => l($value, 'node/' . $key),
      );
    }
  }

  //Prepare reference to another asset
  $vars['references'] = array();
  foreach ($vars['field_asset_node_reference'] as $reference) {
    if (is_array($reference)) {
      $nid = $reference['value']['field_asset_node_reference_node'][0]['nid'];
    }
    if (!empty($nid)) {
      /*
      $relationship = $reference['value']['field_asset_node_relationship'][0]['value'];
      //Retrieve the label of relationship type
      $sql = " SELECT global_settings FROM content_node_field WHERE field_name = '%s' AND type = 'text' ";
      $global_settings = unserialize(db_result(db_query($sql, 'field_asset_node_relationship')));
      $allowed_values_explode = explode("\n", $global_settings['allowed_values']);
      foreach ($allowed_values_explode as $allowed_value) {
        $allowed_value_explode = explode("|", $allowed_value);
        if ($allowed_value_explode[0] == $relationship) {
          $relationship_value = trim($allowed_value_explode[1]);
        }
      }
      $tab_links[$relationship]['label'] = $relationship_value;
       * 
       */
      //$vars['references'][] = l($relationship_value, 'node/' . $nid);
      $link_title = strip_tags($reference['view']);
      //replace multi spaces with single space
      $link_title = preg_replace('!\s+!', ' ', $link_title);
      $link_label = substr(trim($link_title), 0,26) . "..";
      $vars['references'][] = l($link_label, 'node/' . $nid, array("attributes" => array("title" => $link_title)));
    }
  }
  /*
  foreach ($tab_links as $id => $value) {
    $ids = implode(', ', $value['ids']);
    $vars['field_asset_node_reference_' . $id]['value'] = implode(', ', $value['value']);
    $vars['field_asset_node_reference_' . $id]['label'] = $value['label'];
    $vars['field_asset_node_reference_' . $id]['value'] .= "<div id=field_asset_node_reference_'{$id}_button' style='display:inline;'>" .
      ctools_ajax_image_button(
        isa_ajax_get_image('collapsed'),
        "isa_ajax/nojs/teaser_expand/field_asset_node_reference_{$id}/{$ids}/lang={$lang_req}",
        t('expand')) .
      '</div>';
    $vars['fields'][] = 'field_asset_node_reference_' . $id;
  }
*/
  //Prepare taxonomies
  $include = array(
    variable_get("country_vid", NULL),
    //variable_get("domains_vid", NULL),
    //variable_get("keywords_vid", NULL),
    variable_get("asset_type_vid", 68),
    variable_get("interoperability_level_vid", 67),
    variable_get("status_vid", 69),
    variable_get("solution_category_vid", 84),
  );
  isa_toolbox_create_taxonomy_list($node, $include, FALSE, TRUE);
  
  foreach ($node->taxonomy_terms as $vocab => $terms) {
    $voc = str_replace(' ', '', strtolower($vocab));
    $vars['taxonomy_terms_' . $voc]['label'] = $vocab;
    $vars['taxonomy_terms_' . $voc]['value'] = $terms;
    $vars['fields'][] = 'taxonomy_terms_' . $voc;
  }

  $include = array(
    variable_get("domains_vid", NULL),
    variable_get("keywords_vid", NULL),
  );
  isa_toolbox_create_taxonomy_list($node, $include, TRUE);
  foreach ($node->taxonomy_terms as $vocab => $terms) {
    $voc = str_replace(' ', '', strtolower($vocab));
    $vars['taxonomy_terms_' . $voc]['label'] = $vocab;
    $vars['taxonomy_terms_' . $voc]['value'] = $terms;
    $vars['fields'][] = 'taxonomy_terms_' . $voc;
  }


  //Prepare flag
  $flags = flag_get_counts('node', $node->nid);
  if (isset($flags['i_use_this_asset'])) {
    if ($flags['i_use_this_asset'] == 1) {
      $vars['i_use_this_asset'] = '<strong>1</strong> ' . t('person uses this interoperability solution');
    } else {
      $vars['i_use_this_asset'] = '<strong>' . $flags['i_use_this_asset'] . '</strong> ' . t('people use this interoperability solution');
    }
  }

  // https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-732
  // disclaimer to be included at the page bottom of every (federated) asset release (in read mode)
  if (isset($vars['repository_origin'])) {
    $disclaimer = variable_get('group_federated_asset_release_disclaimer_information', FALSE);
    if ($disclaimer) {
      $vars['disclaimer'] = check_markup(token_replace($disclaimer, 'global'), FILTER_FORMAT_FULL, FALSE);
    }
  }
  // https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-744
  // insert a new field with the date of creation, before field_asset_last_modification, so that they will appear in that order in the page
  _joinup_add_field_date_created($vars);

  //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-950
  //Count external downloads in asset release
  if('asset_release' == $node->type){//Add javascript to count in the asset_release views
      $settings = array(
        'isa_private_files' => array(
          'linkClass' => 'asset-external-download',
          'target' => base_path() . 'system/count_asset_external_downloads',
          )
        );
      drupal_add_js($settings, 'setting');
      drupal_add_js(drupal_get_path('module', 'isa_private_files') . '/js/count_asset_release_download.js');
  }
}

/**
 * Override or insert variables into the "node-profile.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_profile(&$vars) {
  $node = $vars['node'];
  $vars['kudos'] = l(userpoints_get_current_points($vars['uid']), isa_toolbox_get_kudos_page(),
    array('attributes' => array('title' => t('What are kudos?'), 'class' => 'popups')));
  $vars['created'] = format_interval(time() - $vars['node']->created, 2);
  
  foreach ($node->field_country as $country) {
    $countries[] = $country['view'];
  }
  if(sizeof($countries)>1&&!empty($countries)){
    $countries = implode(", ",$countries);
  }else{
    $countries = $countries[0];
  }
  $vars['countries'] = $countries;
  
  foreach ($node->field_languages_fields as $language) {
    $languages[] = $language['view'];
  }
  if(sizeof($languages)>1&&!empty($languages)){
    $languages = implode(", ",$languages);
  }else{
    $languages = $languages[0];
  }
  $vars['languages'] = $languages;
  
  foreach ($node->field_profile_sector as $sector) {
    $sectors[] = $sector['view'];
  }
  if(sizeof($sectors)>1&&!empty($sectors)){
    $sectors = implode(", ",$sectors);
  }else{
    $sectors = $sectors[0];
  }
  $vars['sectors'] = $sectors;
  
  isa_toolbox_get_user_country($vars['node']);
  $completion = db_result(db_query("SELECT field_completed_profile_value FROM {content_type_profile} WHERE nid = '%s'", $node->nid));
  $vars['completion'] = $completion;
  $include = array(
    variable_get("languages_vid", NULL),
    variable_get("domains_vid", NULL),
    variable_get("keywords_vid", NULL));
  isa_toolbox_create_taxonomy_list($node, $include);
  $vars['taxonomy_terms'] = $node->taxonomy_terms;
  drupal_set_title(check_plain($vars['node']->field_firstname[0]['value'] . ' ' . $vars['node']->field_lastname[0]['value']));
}

/**
 * Override or insert variables into the "node-project.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_project_project(&$vars) {
  global $base_url;
  $node = $vars['node'];
  //og remove $vars['submitted'] (og.module l 449), but is useful
  $vars['submitted'] = theme('node_submitted', $node);
  // https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-600
  // Obtain name of principal mailing list, if any
  $mailing_list = isa_ml_get_principal_mailing_list($node->nid);
  $vars['mailing_list'] = $mailing_list ? isa_toolbox_protect_email(isa_ml_get_principal_mailing_list($node->nid) . '@' . variable_get('isa_ml_domain', NULL)) : FALSE;

  // Maven url
  if ($vars['field_project_display_maven'][0]['value']) {
    $maven_url = str_replace('@shortname@', $node->project['uri'], variable_get('maven_url', $base_url . '/nexus'));
    $vars['maven_url'] = check_plain($maven_url);
  }
  // SVN
  if ($vars['field_project_display_svn']['0']['value']) {
    $svn_url = str_replace('@shortname@', $node->project['uri'], variable_get('svn_url', $base_url . '/joinup-svn/@shortname@'));
    $vars['svn_url'] = check_plain($svn_url);
  }

  // WebDAV & WebDir
  $web_directory = isa_toolbox_get_web_directory(array('nid' => $node->nid));
  if ($web_directory) {
    if (og_is_group_member($node->nid, TRUE)) {
      $webdav_url = str_replace('@shortname@', $node->project['uri'], variable_get('webdav_url', $base_url . '/joinup-webdav/@shortname@'));
      $vars['webdav_url'] = check_plain($webdav_url);
    }
    if (isset($web_directory[0]) && $web_directory[0]->public_access == WEB_DIR_PUBLIC_ACCESS_ENABLED) {
      if ($web_directory[0]->state == WEB_DIR_STATUS_TODO) {
        $vars['webdir_url'] = t('The web directory is being created');
      }
      elseif ($web_directory[0]->state == WEB_DIR_STATUS_DONE) {
        $webdir_url = str_replace('@shortname@', $node->project['uri'], variable_get('webdir_url', $base_url . '/site/@shortname@'));
        $vars['webdir_url'] = check_plain($webdir_url);
      }
      elseif ($web_directory[0]->state == WEB_DIR_STATUS_FAILED) {
        if (og_is_group_admin($node)) {
          $vars['webdir_url'] = t('The web directory creation is failed, you can <a href="@contact-form">contact</a> the site administrator',
            array('@contact-form' => url('contact')));
        }
      }
    }
  }
  if ($node->tnid == 0) {
    $flags = flag_get_counts('node', $node->nid);
  } else {
    $flags = flag_get_counts('node', $node->tnid);
  }
  if (isset($flags['i_use_this_project'])) {
    if ($flags['i_use_this_project'] == 1) {
      $vars['i_use_this_project'] = '<strong>1</strong> ' . t('person use this project');
    }
    else {
      $vars['i_use_this_project'] = '<strong>' . $flags['i_use_this_project'] . '</strong> ' . t('people use this project');
    }
  }

  $vars['type'] = check_plain(node_get_types('name', $node->type));

  // Get group users
  $vars['og_members'] = _joinup_get_og_members($node->nid);

  if(!empty($node->field_project_adms_sw_project)){
    $software = node_load($node->field_project_adms_sw_project[0]['nid']);
  }
  if($software){
    if (!empty($software->field_sw_project_bug_database)){
	  foreach($software->field_sw_project_bug_database as $bug_database){
	    $vars['bug_database'][] = check_plain($bug_database['value']);
	  }
	}
	if (!empty($software->field_sw_project_wiki)){
	  foreach($software->field_sw_project_bug_database as $wiki){
	    $vars['wiki'][] = check_plain($wiki['value']);
	  }
	}
	if (!empty($software->field_sw_project_repository[0]['nid'])){
	  $repository = node_load($software->field_sw_project_repository[0]['nid']);
	  $vars['repository'] = l($repository->title, $repository->path);
	}
	if (!empty($software->field_sw_project_agents[0]['nid'])){
	  $agent = node_load($software->field_sw_project_agents[0]['nid']);
	  $vars['agent'] = l($agent->title, $agent->path);
	}
	if (!empty($software->field_sw_release[0]['nid'])){
	  $release = node_load($software->field_sw_release[0]['nid']);
	  $vars['release'] = l($release->title, $release->path);
	}
	$vocabulary_include = array(
	  80,//topic
	  81,//user_interface
	  79,//programming_language
	  78,//intended_audience
	  83, //operating_system
	  77, //locale
	  82, //status
	  73, //file format
	  84, //solution category
	);
	isa_toolbox_create_taxonomy_list($software, $vocabulary_include);
    // sort the taxonomy terms
    foreach ($vocabulary_include as $key => $value) {
      $vocab = taxonomy_vocabulary_load($value);
      if (isset($software->taxonomy_terms[$vocab->name])) {$test[$value] = $software->taxonomy_terms[$vocab->name];
        $vars['taxonomy_terms'][check_plain($vocab->name)] = $software->taxonomy_terms[$vocab->name];
      }
    }
	
  }
  
  
  if ($node->group_type == ISA_SOFTWARE_TYPE) {
    _joinup_preprocess_software($vars);
  } elseif ($node->group_type == ISA_ASSET_TYPE) {
    _joinup_preprocess_asset($vars);
  }
}

function _joinup_preprocess_software(&$vars) {
  //global $base_url;
  $node = $vars['node'];
  $default_language = variable_get('language_default', NULL);
  $default_language = $default_language->language;
  if ($node->language != $default_language) {
    $vars['node']->content['og_mission']['#value'] = str_replace('<p>',
      "<p lang='" . $node->language . "'>", $vars['node']->content['og_mission']['#value']);
    $vars['node']->field_project_soft_features['0']['value'] = str_replace('<p>',
      "<p lang='" . $node->language . "'>", $vars['node']->field_project_soft_features['0']['value']);
    $vars['node']->field_project_soft_future_plans['0']['value'] = str_replace('<p>',
      "<p lang='" . $node->language . "'>",
      $vars['node']->field_project_soft_future_plans['0']['value']);
    $vars['node']->field_project_soft_get_involved['0']['value'] = str_replace('<p>',
      "<p lang='" . $node->language . "'>",
      $vars['node']->field_project_soft_get_involved['0']['value']);
    $vars['node']->field_project_soft_public_admin['0']['value'] = str_replace('<p>',
      "<p lang='" . $node->language . "'>",
      $vars['node']->field_project_soft_public_admin['0']['value']);
  }
  
  // use node-software.tpl.php for displays software
  array_push($vars['template_files'], 'node-' . $node->group_type);
  $vocabulary_include = array(
    variable_get("license_vid", NULL),
    variable_get("categories_vid", NULL),
  );
  
  
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include);
  // sort the taxonomy terms
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
  $more_links = array();
  foreach ($node->field_project_soft_more_lin as $key => $value) {
    if (!empty($value['view'])) {
      $more_links[] = check_plain($value['view']);
    }
  }
  $vars['more_links'] = $more_links;

  if (user_access('administer nodes')) {
    $vars['edit_link'] = l('Edit this software', "node/{$node->nid}/edit");
  }
  //joinup_set_terms_in_vars($vars, $node);
}

function _joinup_preprocess_asset(&$vars) {
  //global $base_url;
  $node = $vars['node'];
  $default_language = variable_get('language_default', NULL);
  $default_language = $default_language->language;
  if ($node->language != $default_language) {
    $vars['node']->content['og_mission']['#value'] = str_replace('<p>',
      "<p lang='" . $node->language . "'>", $vars['node']->content['og_mission']['#value']);
  }
  // use node-asset.tpl.php for displayes asset temlplate
  array_push($vars['template_files'], 'node-' . $node->group_type);

  $vocabulary_include = array(
    variable_get("license_vid", NULL),
    variable_get("represented_countries_vid", NULL),
	variable_get("represented_countries_vid", NULL),
    variable_get("categories_vid", NULL),
	variable_get('asset_status_vid', NULL),
  );
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include);

  // sort the taxonomy terms
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if ($vocab && isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
  if (user_access('administer nodes')) {
    $vars['edit_link'] = l('Edit this asset', "node/{$node->nid}/edit");
  }
}

/**
 * Override or insert variables into the "node-federated_project.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_federated_project(&$vars) {
  $node = $vars['node'];
  $default_language = variable_get('language_default', NULL);
  $default_language = $default_language->language;
  if ($node->tnid == 0) {
    $flags = flag_get_counts('node', $node->nid);
  } else {
    $flags = flag_get_counts('node', $node->tnid);
  }
  if (isset($flags['i_use_this_project'])) {
    $vars['i_use_this_project'] = '<strong>' . $flags['i_use_this_project'] . '</strong> ' . t('people use this project');
  } else {
    $vars['i_use_this_project'] = t('No people using this project');
  }
  if ($node->language != $default_language) {
    $vars['node']->content['body']['#value'] = str_replace('<p>',
      "<p lang='" . $node->language . "'>", $vars['node']->content['body']['#value']);
    $vars['node']->field_project_soft_features['0']['value'] = str_replace('<p>',
      "<p lang='" . $node->language . "'>", $vars['node']->field_project_soft_features['0']['value']);
    $vars['node']->field_project_soft_future_plans['0']['value'] = str_replace('<p>',
      "<p lang='" . $node->language . "'>",
      $vars['node']->field_project_soft_future_plans['0']['value']);
    $vars['node']->field_project_soft_get_involved['0']['value'] = str_replace('<p>',
      "<p lang='" . $node->language . "'>",
      $vars['node']->field_project_soft_get_involved['0']['value']);
    $vars['node']->field_project_soft_public_admin['0']['value'] = str_replace('<p>',
      "<p lang='" . $node->language . "'>",
      $vars['node']->field_project_soft_public_admin['0']['value']);
  }

  $vars['translations'] = isa_toolbox_get_translations_list($node->nid);
  $vocabulary_include = array(
    variable_get("development_status_vid", NULL),
    variable_get("intended_audience_vid", NULL),
    variable_get("license_vid", NULL),
    variable_get("natural_language_vid", NULL),
    variable_get("operating_system_vid", NULL),
    variable_get("programming_language_vid", NULL),
    variable_get("keywords_vid", NULL),
  );
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include);
  // sort the taxonomy terms
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
  $more_links = array();

  foreach ($node->field_project_soft_more_links as $key => $value) {
    if (!empty($value['view'])) {
      $more_links[] = check_plain($value['view']);
    }
  }
  $vars['more_links'] = implode(", ", $more_links);
}

///**
// * Override or insert variables into the "node-federated_forge.tpl.php" templates.
// *
// * @param array $vars
// *   An array of variables to pass to the theme template.
// *
// * @see _joinup_preprocess_node
// */
//function _joinup_preprocess_federated_forge(&$vars) {
//  $node = $vars['node'];
//  // Reload the cached node
//  $url = $node->field_forges_homepage['0']['url'];
//  if (substr($url, 0, 5) != 'http:') {
//    $url = 'http://' . $url;
//  }
//  //$vars['homepage']['url'] = l($node->field_forges_homepage['0']['title'], $url, array('external' => TRUE, 'abbsolute' => TRUE));
//}

/**
 * Override or insert variables into the "node-community.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_community(&$vars) {



  $node = $vars['node'];
  //$community_shortname = $vars['field_community_short_name']['0']['value'];
  $vars['submitted'] = theme('node_submitted', $node);
  // https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-600
  // Obtain name of principal mailing list, if any
  $mailing_list = isa_ml_get_principal_mailing_list($node->nid);
  $vars['mailing_list'] = $mailing_list ? isa_toolbox_protect_email(isa_ml_get_principal_mailing_list($node->nid) . '@' . variable_get('isa_ml_domain', NULL)) : FALSE;

  if (user_access('administer nodes')) {
    $vars['edit_link'] = l('Edit this community', "node/{$node->nid}/edit");
  }
  $vocabulary_include = array(
    variable_get("domains_vid", NULL),
    variable_get("keywords_vid", NULL),
  );
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include);

  // sort the taxonomy terms
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }

  //Get group users
  $vars['og_members'] = _joinup_get_og_members($node->nid);
}

/**
 * Override or insert variables into the "node-topic.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_topic(&$vars) {
  $node = $vars['node'];
  $profile = content_profile_load('profile', $vars['uid']);
  $picture = $profile->picture;
  $vars['picture'] = $picture;
  $vars['user_company_name'] = check_plain($profile->field_company_name['0']['value']);
  $vars['postnumber'] = isa_toolbox_get_users_topics_count($node->uid);

  //set the taxonomy terms for user
  isa_toolbox_create_taxonomy_list($profile, array(variable_get('country_vid', NULL)));
  $country = taxonomy_vocabulary_load(variable_get('country_vid', NULL));
  if (isset($profile->taxonomy_terms[$country->name])) {
    $vars['user_countries'] = $profile->taxonomy_terms[$country->name];
  }
  //set the taxonomy terms for community
  isa_toolbox_create_taxonomy_list($node,
    array(variable_get('domains_vid', NULL), variable_get('forum_vid', NULL), variable_get('languages_vid',
      NULL)));
  $domains = taxonomy_vocabulary_load(variable_get('domains_vid', NULL));
  $vars['domains'] = $node->taxonomy_terms[$domains->name];
  $languages = taxonomy_vocabulary_load(variable_get('languages_vid', NULL));
  $vars['languages'] = !empty($node->taxonomy_terms[$languages->name]) ? $node->taxonomy_terms[$languages->name] : NULL;

  $forum = taxonomy_vocabulary_load(variable_get('forum_vid', NULL));
  if (isset($node->taxonomy_terms[$forum->name])) {
    $gid = $vars['og_groups'];
    $gid = array_shift($gid);
    $group = node_load($gid);

    foreach ($node->taxonomy as $tid => $value) {
      if ($value->vid == variable_get('forum_vid', NULL)) {
        $forum_name = $value->name;
      }
    }
    $group_short_name = isa_links_get_group_short_name($group);
    $vars['forum'] = l($node->taxonomy_terms[$forum->name],
      "$group->group_type/$group_short_name/forum/$forum_name");
  }
  $keywords = taxonomy_vocabulary_load(variable_get('keywords_vid', NULL));
  if (isset($node->taxonomy_terms[$keywords->name])) {
    $vars['keywords'] = $node->taxonomy_terms[$keywords->name];
  }
  //set the taxonomy term for topic
  isa_toolbox_create_taxonomy_list($node);
  $domains = taxonomy_vocabulary_load(variable_get('domains_vid', NULL));
  if (isset($node->taxonomy_terms[$domains->name])) {
    $vars['domains'] = $node->taxonomy_terms[$domains->name];
  }
  $languages = taxonomy_vocabulary_load(variable_get('languages_vid', NULL));
  if (isset($node->taxonomy_terms[$languages->name])) {
    $vars['languages'] = $node->taxonomy_terms[$languages->name];
  }
  //set the taxonomy term for user
  //joinup_set_terms_in_vars($vars, $node);
  //add highlight link
  $vars['highlight_link'] = !empty($node->links['hl_link']) ? theme_links($node->links['hl_link']) : NULL;
  //add subscription link
  $vars['subscription_link'] = !empty($node->links['subscription_link']) ? theme_links($node->links['subscription_link']) : NULL;
  //add statistics reads
  $statistic = statistics_get($node->nid);
  $vars['statistic_counter'] = $statistic['totalcount'];
  //$vars['highlight_link'] =  $node->links['statistics_counter']['title'];

  $group = $vars['og_groups'];
  $group_keys = array_keys($group);
  $group_id = array_shift($group_keys);
  if (og_is_group_member($group_id, FALSE, $vars['uid'])) {
    $user = user_load($vars['uid']);
    $group = node_load($group_id);
    if ($group->uid == $vars['uid']) {
      $joined_group = $group->created;
    } else {
      $joined_group = $user->og_groups[$group_id]['created'];
    }
    $vars['joined_group'] = date("d/m/Y", $joined_group);
  }
}

/**
 * Override or insert variables into the "node-document.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_document(&$vars) {
  $node = $vars['node'];

  // Document categories
  $vocabulary_include = array(
    variable_get("nature_of_documentation_vid", NULL),
    variable_get("document_license_vid", NULL),
    variable_get("country_vid", NULL),
    variable_get("domains_vid", NULL),
    variable_get("keywords_vid", NULL)
  );
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include, TRUE);
  // Sort the taxonomy terms
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
  // Document Information
  $vars['document_info'] = array(
    'Publication date' => date('d F Y', strtotime($node->field_publication_date['0']['value'])),
    'Publisher' => $node->field_publisher['0']['safe'],
    'ISBN Number' => $node->field_isbn['0']['safe'],
    'Description of license' => $node->field_description_of_license['0']['safe']
  );
  $vars['original_url']['url'] = check_plain($node->field_original_url['0']['url']);
  $vars['original_url']['title'] = check_plain($node->field_original_url['0']['title']);
  $vars['email_contact'] = $node->field_email['0']['safe'];
}

/**
 * Override or insert variables into the "node-document.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_presentation(&$vars) {
  $node = $vars['node'];

  // Document categories
  $vocabulary_include = array(
    variable_get("nature_of_documentation_vid", NULL),
    variable_get("document_license_vid", NULL),
    variable_get("country_vid", NULL),
    variable_get("domains_vid", NULL),
    variable_get("keywords_vid", NULL)
  );
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include, TRUE);
  // Sort the taxonomy terms

  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);


    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
  // Document Information
  $vars['presentation_info'] = array(
    'Publication date' => date('d F Y', strtotime($node->field_publication_date['0']['value'])),
    'Publisher' => $node->field_publisher['0']['safe']
  );
  $vars['original_url']['url'] = check_plain($node->field_original_url['0']['url']);
  $vars['original_url']['title'] = check_plain($node->field_original_url['0']['title']);
  $vars['email_contact'] = $node->field_email['0']['safe'];
}

/**
 * Override or insert variables into the "node-case.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _isa_icp_preprocess_node
 */
function _joinup_preprocess_case(&$vars) {
  $node = $vars['node'];

  // Document categories
  $vocabulary_include = array(
    variable_get("domains_vid", NULL),
  );
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include);
  // Sort the taxonomy terms
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
}

/**
 * Override or insert variables into the "node-case_epractice.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_case_epractice(&$vars) {
  $node = $vars['node'];
  foreach($node->taxonomy as $key => $value) {
    $vocabulary_include[] = $value->vid;
  }
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include);
  // Sort the taxonomy terms
  if(isset($node->field_case_sector)&&!empty($node->field_case_sector)){
    foreach($node->field_case_sector as $sector){
	  $sectors[] = $sector['view'];
	}
	if(sizeof($sectors)>1){
	  $sectors = implode($sectors,", ");
	}else{
	  $sectors = $sectors[0];
	}
	$vars['taxonomy_terms']['Sector'] = $sectors;
  }
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }

  // Case Information
  
  if(isset($node->field_acronym['0']['value'])&&!empty($node->field_acronym['0']['value'])){
    $acronym=$node->field_acronym['0']['value'];
  }
  if(isset($node->field_presentation_url['0']['url'])&&!empty($node->field_presentation_url['0']['url'])){
    $presentation_url= (!empty($node->field_presentation_url['0']['title'])) ? '<a href="' . $node->field_presentation_url['0']['url'] . '">' . $node->field_presentation_url['0']['title'] . '</a>'
        : '<a href="' . $node->field_presentation_url['0']['url'] . '">' . $node->field_presentation_url['0']['url'] . '</a>';
  }
  if(isset($node->field_website_url['0']['url'])&&!empty($node->field_website_url['0']['url'])){
    $website_url=(!empty($node->field_website_url['0']['title'])) ? '<a href="' . $node->field_website_url['0']['url'] . '">' . $node->field_website_url['0']['title'] . '</a>'
        : '<a href="' . $node->field_website_url['0']['url'] . '">' . $node->field_website_url['0']['url'] . '</a>';
  }
  if(isset($node->field_city_region['0']['value'])&&!empty($node->field_city_region['0']['value'])){
    $city_region=$node->field_city_region['0']['value'];
  }
  if(isset($node->field_start_end_date['0']['value'])&&!empty($node->field_start_end_date['0']['value'])){
    $start_date=date('d F Y', strtotime($node->field_start_end_date['0']['value']));
  }
  if(isset($node->field_start_end_date['0']['value2'])&&!empty($node->field_start_end_date['0']['value2'])){
    $end_date=date('d F Y', strtotime($node->field_start_end_date['0']['value2']));
  }
  if(isset($node->field_operational_date['0']['value'])&&!empty($node->field_operational_date['0']['value'])){
    $operational_date=date('d F Y', strtotime($node->field_operational_date['0']['value']));
  }
  $vars['case_info'] = array(
    'Acronym' => $acronym,
    'Presentation URL' => $presentation_url,
    'Website URL' => $website_url,
    'City/Region' => $city_region,
    'Start date' => $start_date,
    'End date' => $end_date,
    'Operational date' => $operational_date
  );
  // Case long text information
  $vars['case_long_text_info'] = array(
    'Policy context' => array('class' => 'policy', 'text' => check_markup($node->field_policy_context['0']['value'], FILTER_FORMAT_FULL, FALSE)),
    'Target users' => array('class' => 'target-user', 'text' => check_markup($node->field_target_users['0']['value'], FILTER_FORMAT_FULL, FALSE)),
    'Description of target users and groups' => array('class' => 'target-desc', 'text' => check_markup($node->field_desc_target_users_groups['0']['value'], FILTER_FORMAT_FULL, FALSE)),
    'Description of the way to implement the initiative' => array('class' => 'implementation-desc', 'text' => check_markup($node->field_desc_implementation['0']['value'], FILTER_FORMAT_FULL, FALSE)),
    'Technology solution' => array('class' => 'tech-solution', 'text' => check_markup($node->field_tech_solution['0']['value'], FILTER_FORMAT_FULL, FALSE)),
    'Main results, benefits and impacts' => array('class' => 'benefits', 'text' => check_markup($node->field_main_results['0']['value'], FILTER_FORMAT_FULL, FALSE)),
    'Return on investment description' => array('class' => 'investment-desc', 'text' => check_markup($node->field_roi_desc['0']['value'], FILTER_FORMAT_FULL, FALSE)),
    'Track record of sharing' => array('class' => 'track-record', 'text' => check_markup($node->field_track_record_sharing['0']['value'], FILTER_FORMAT_FULL, FALSE)),
    'Lessons learnt' => array('class' => 'lessons-learnt', 'text' => check_markup($node->field_lessons_learnt['0']['value'], FILTER_FORMAT_FULL, FALSE))
  );
}

/**
 * Override or insert variables into the "node-news.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see joinup_preprocess_node
 */
function _joinup_preprocess_project_issue(&$vars) {
  $node = $vars['node'];
  $project = node_load(array('nid' => $node->project_issue['pid'], 'type' => 'project_project'));
  $release->nid = $node->project_issue['rid'];
  if (module_exists('project_release')) {
    $release = project_release_load($release);
  }
  $assigned = ($node->project_issue['assigned'] && ($account = user_load(array('uid' => $node->project_issue['assigned']))) ? check_plain($account->name) : t('Unassigned'));

  $current_data = array();
  $current_data['pid'] = array(
    'label' => t('Project'),
    'current' => check_plain($project->title),
  );
  if (!empty($release->project_release['version'])) {
    $current_data['rid'] = array(
      'label' => t('Version'),
      'current' => check_plain($release->project_release['version']),
    );
  }
  $current_data['component'] = array(
    'label' => t('Component'),
    'current' => check_plain($node->project_issue['component']),
  );
  $current_data['category'] = array(
    'label' => t('Category'),
    'current' => project_issue_category($node->project_issue['category'], 0, $node->project_issue['pid']),
  );
  $current_data['priority'] = array(
    'label' => t('Priority'),
    'current' => project_issue_priority($node->project_issue['priority']),
  );
  $current_data['assigned'] = array(
    'label' => t('Assigned'),
    'current' => $assigned,
  );
  $current_data['sid'] = array(
    'label' => t('Status'),
    'current' => check_plain(project_issue_state($node->project_issue['sid'])),
  );
  $current_data['nid'] = array(
    'label' => t('Identifier'),
    'current' => $node->project_issue['nid'],
  );
  $vars['issue_data'] = $current_data;
}

function _joinup_preprocess_blog(&$vars) {
  _joinup_preprocess_news($vars);
}

function _joinup_preprocess_news(&$vars) {
  $node = $vars['node'];

  //prepare taxonomy
  $vocabulary_include = array(
    variable_get("country_vid", NULL),
    variable_get("domains_vid", NULL),
    variable_get("keywords_vid", NULL),
  );
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include, TRUE);
  //sort taxonomy
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }

  // prepare author name
  $vars['author'] = theme('username', $node);

  //prepare date created
  $vars['created_view'] = date('d/m/Y - G:i', $node->created);

	//https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-123
	// A user should be able to report an abuse
	$gid = variable_get('current_group', isa_toolbox_get_community_nid());
	if ($node->type == ISA_NEWS_TYPE) {
		if($gid>0){ // this news belongs to a group
			$node->link_abuse = true;
		}
	}


}

/**
 * Override or insert variables into the "node-factsheet.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_factsheet(&$vars) {
  $node = $vars['node'];
  //prepare taxonomy
  $vocabulary_include = array(
    variable_get("domains_vid", NULL),
    variable_get("keywords_vid", NULL),
    variable_get("factsheet_topic_vid", NULL),
    variable_get("country_vid", NULL),
  );
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include);
  // sort the taxonomy terms
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
  //prepare link
//  $vars['statistics_counter'] = $node->links['statistics_counter']['title'];
//  unset($node->links['statistics_counter']);
//  $vars['links'] = theme_links($node->links);
}

/**
 * Override or insert variables into the "node-event.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_event
 */
function _joinup_preprocess_event(&$vars) {
  $node = $vars['node'];

  //prepare link
  $vars['links'] = theme_links($node->links);
  // Document categories
  $vocabulary_include = array(
    variable_get("country_vid", NULL),
    variable_get("keywords_vid", NULL),
    variable_get("domains_vid", NULL),
    variable_get("scope_vid", NULL),
    variable_get("free_event_vid", NULL),
    variable_get("open_event_vid", NULL),
    variable_get("organisation_type_vid", NULL)
  );

  isa_toolbox_create_taxonomy_list($node, $vocabulary_include, TRUE);
  $vars['taxonomy_terms_links'] = array();
  // sort the taxonomy terms
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
  $display_website = ($node->field_event_website['0']['title'] != "" && $node->field_event_website['0']['url'] != "");

  // event Information
  $vars['event_info'] = array(
    'Start date' => isa_toolbox_get_timezone_date($node->field_event_dates['0']['value'], $node->field_event_dates['0']['timezone']),
    'End date' => isa_toolbox_get_timezone_date($node->field_event_dates['0']['value2'], $node->field_event_dates['0']['timezone']),
    'City' => $node->field_event_city['0']['safe'],
    'GMap location' => $node->field_event_gmap_location['0']['safe'],
    'Organiser' => $node->field_event_organiser['0']['safe'],
    //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-747
    //Add query to params
    'Website' => $display_website ? l($node->field_event_website['0']['title'],
        $node->field_event_website['0']['url'], array('query' =>  $node->field_event_website['0']['query'])) : "",
    'Contact email' => isa_toolbox_protect_email($node->field_event_contact_email['0']['value']),
  );
  // Case long text information
  $vars['event_long_text_info'] = array(
    'Venue' => array('class' => 'policy', 'text' => check_markup($node->field_event_venue['0']['value'], FILTER_FORMAT_FULL, FALSE)),
    'Address/Location' => array('class' => 'policy', 'text' => check_markup($node->field_event_address_location['0']['value'], FILTER_FORMAT_FULL, FALSE)),
    'Agenda' => array('class' => 'policy', 'text' => check_markup($node->field_event_agenda['0']['value'], FILTER_FORMAT_FULL, FALSE)),
    'Expected participants' => array('class' => 'policy', 'text' => check_markup($node->field_event_exp_participants['0']['value'], FILTER_FORMAT_FULL, FALSE)),
    'Fees description' => array('class' => 'policy', 'text' => check_markup($node->field_event_fees_description['0']['value'], FILTER_FORMAT_FULL, FALSE)),
  );
}

function _joinup_preprocess_newsletter(&$vars) {
  $node = $vars['node'];
  $vocabulary_include = array(
    variable_get("newsletter_vid", NULL),
    variable_get("domains_vid", NULL),
  );
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include);
  // Sort the taxonomy terms
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
}

/**
 * Override or insert variables into the "node-video.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_video
 */
function _joinup_preprocess_video(&$vars) {
  $node = $vars['node'];
  $vars['group_description'] = $node->content['body']['#value'];
  //prepare link
  $vars['links'] = theme_links($node->links);
  // Video categories
  $vocabulary_include = array(
    variable_get("domains_vid", NULL),
    variable_get("keywords_vid", NULL)
  );
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include, TRUE);
  // Sort the taxonomy terms
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
  // Video Information
  $vars['video'] = $node->field_video_link_to_video[0]['view'];
}

function _joinup_preprocess_wiki(&$vars) {
  $node = $vars['node'];
  $path_alias = explode('/', $vars['node']->path);
  if ($path_alias[1] == 'license-wizard')
    unset($vars['node']->content['fivestar_widget']);
  $vars['description'] = $vars['node']->content['body']['#value'];

  //prepare taxonomy
  $vocabulary_include = array(
    variable_get("country_vid", NULL),
    variable_get("domains_vid", NULL),
    variable_get("keywords_vid", NULL),
  );
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include, TRUE);
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
}

function _joinup_preprocess_project_release(&$vars) {
  $node = $vars['node'];
  //prepare taxonomy
  $vocabulary_include = array(
    variable_get("package_vid", 66),
  );
  isa_toolbox_create_taxonomy_list($node, $vocabulary_include, FALSE);
  foreach ($vocabulary_include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
}

/**
 * Override or insert variables into the "node-repository.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_repository(&$vars) {
  global $user;
  unset($vars['title']);
  $node = $vars['node'];

  $vars['submitted'] = theme('node_submitted', $node);

  //Prepare the logo
  if (isset($node->field_repository_logo['0'])) {
    $path = $node->field_repository_logo['0']['filepath'];
  } else {
    $picture = isa_toolbox_default_imagefield('field_repository_logo', ISA_REPOSITORY_TYPE);
    $path = $picture['filepath'];
  }
  $vars['picture'] = theme_imagecache('community_logo', $path, $node->title);

  $exclude = array(
    "field_repository_description",
    "field_repository_logo",
    "field_repository_name");
  //Don't display the fields related to harvest mode, if the mode is manual.
  if (!$vars['field_repository_harvestmethod'][0]['value']) {
    $exclude[] = 'field_repository_harvestfreq';
    $exclude[] = 'field_repository_harvesturl';
  }

  $language_textfields = array(array('description', 'field_repository_description', 'body'),
    array('field_name', 'field_repository_name', 'title'));
  _joinup_prepare_template($language_textfields, $vars, $exclude);

  //Prepare the report link
  $report_path = file_directory_path() . "/import_metadata/report_{$node->nid}.log";
  if (file_exists($report_path) && (user_access('administer nodes') || $node->uid == $user->uid)) {
    $vars['report_file']['label'] = 'Report file';
    $vars['report_file']['value'] = l('Report file', $report_path,
      array(
      'query' => array(
        'time' => time(),
      )));
    array_unshift($vars['fields'], 'report_file');
  }

  //Prepare taxonomies
  $include = array(
    variable_get("geographic_coverage_vid", NULL), // country_vid
    //variable_get("domains_vid", NULL),
  );

  isa_toolbox_create_taxonomy_list($node, $include, FALSE, TRUE);
  foreach ($node->taxonomy_terms as $vocab => $terms) {
    $voc = str_replace(' ', '', strtolower($vocab));
    $vars['taxonomy_terms_' . $voc]['label'] = $vocab;
    $vars['taxonomy_terms_' . $voc]['value'] = $terms;
    $vars['fields'][] = 'taxonomy_terms_' . $voc;
  }

  $include = array(
    variable_get("domains_vid", NULL),
    variable_get("keywords_vid", NULL),
  );
  isa_toolbox_create_taxonomy_list($node, $include, TRUE);
  foreach ($node->taxonomy_terms as $vocab => $terms) {
    $voc = str_replace(' ', '', strtolower($vocab));
    $vars['taxonomy_terms_' . $voc]['label'] = $vocab;
    $vars['taxonomy_terms_' . $voc]['value'] = $terms;
    $vars['fields'][] = 'taxonomy_terms_' . $voc;
  }
  
  // https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-744
  // insert a new field with the date of creation, before field_asset_last_modification, so that they will appear in that order in the page
  _joinup_add_field_date_created($vars);
  $profile = content_profile_load('profile', $node->uid);

  $vars['owner'] = array(
    'uid' => $node->uid,
    'name' => check_plain($profile->field_firstname[0]['value'] . ' ' . $profile->field_lastname[0]['value'])
  );
}


/**
 * Override or insert variables into the "node-project.tpl.php" templates.
 *
 * @param array $vars
 *   An array of variables to pass to the theme template.
 *
 * @see _joinup_preprocess_node
 */
function _joinup_preprocess_project(&$vars) {
  //global $user;
  $node = $vars['node'];

  $vars['submitted'] = theme('node_submitted', $node);
  $repository = og_get_group_context();
  //Prepare the logo
  if (isset($repository->field_repository_logo['0'])) {
    $path = $repository->field_repository_logo['0']['filepath'];
  } else {
    $picture = isa_toolbox_default_imagefield('field_repository_logo', ISA_REPOSITORY_TYPE);
    $path = $picture['filepath'];
  }
  $vars['picture'] = theme_imagecache('community_logo', $path, $repository->title);

  $exclude = array(
    "field_sw_release",
    'field_sw_project_description'
    );

  $language_textfields = array(
    array(
      'field_name',
      'field_sw_project_name',
       'title'
    ),
    array(
      'description',
      'field_sw_project_description',
      'body')
  );
  _joinup_prepare_template($language_textfields, $vars, $exclude);

  //Prepare taxonomies
  $include = array(
    variable_get("domains_vid", NULL),
    77,//Trove: Translations
    78,//Trove: Intended
    79,//Trove: Programming
    80,//Trove: Topic
    81,//Trove: User
    82,//Trove: Development
    83,//Trove: Operating
  );

  isa_toolbox_create_taxonomy_list($node, $include);
  // sort the taxonomy terms
  foreach ($include as $key => $value) {
    $vocab = taxonomy_vocabulary_load($value);
    if (isset($node->taxonomy_terms[$vocab->name])) {
      $vars['taxonomy_terms'][check_plain($vocab->name)] = $node->taxonomy_terms[$vocab->name];
    }
  }
}

  //Avoid php error in watchdog: Warning: Missing argument 2 for {closure}() in {closure}()
function filter_field ($fieldname) {
      return preg_match('#^field_.*$#', $fieldname);
};

function _joinup_prepare_template($language_textfields, &$vars, $exclude) {
  $node = $vars['node'];
  $node_origin = node_load($node->nid);
  $node_prepare = node_prepare($node_origin);

  // Prepare the multilingualism usage
  $lang_req = isa_toolbox_get_lang_in_url();

  // Prepare the textfields with language
  foreach ($language_textfields as $language_textfield) {
    $label = $language_textfield[0];
    $field_origin = $language_textfield[2];
    foreach ($node->{$language_textfield[1]} as $item) {
      $lang = $item['value']['field_language_textfield_lang'][0]['value'];
      if ($lang == $lang_req) {
        $vars[$label] = $item['value']["field_{$item['type']}_name"][0]['value'];
      }
    }
    if (/*$lang_req == 'en' ||*/ empty($vars[$label])) {
      $vars[$label] = $node_prepare->{$field_origin};
    }
    if ($field_origin == 'title') {
      $title = check_plain($vars[$label]);
      drupal_set_title($title);
      $vars['title'] = $title;
    }
  }

  // Add CTools' javascript to the page.
  ctools_include('ajax');
  ctools_include('modal');
  ctools_modal_add_js();

  $fields = array_keys(get_object_vars($node));
  
  //Get only the fields which are name field_...
  $vars['fields'] = array_filter($fields, "filter_field");
  //Exclude excluded fields.
  $vars['fields'] = array_diff($vars['fields'] , $exclude);

  foreach ($vars['fields'] as $field) {
    $label = content_fields($field);
    $vars[$field]['label'] = $label['widget']['label'];
    if (((!isset($node->{$field}[0]['safe']) || count($node->{$field}[0]['safe']) == 0 )
      && (!isset($node->{$field}[0]['value']))) && (!isset($node->{$field}[0]['fid'])) && (!isset($node->{$field}[0]['view']))) {
	  //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-793
	  //Remove 'To be completed' by an empty string when a fields is missing
      $vars[$field]['value'] = '';
      continue;
    }

    // Prepare the multilingualism usage
    $ids = $links = array();
    $expandable = false;
    foreach ($node->$field as $item) {
      //Get the value corresponding to language
      if (isset($item['type']) && $item['type'] == 'language_textfield') {
        $lang = $item['value']['field_language_textfield_lang'][0]['value'];
        if ($lang == $lang_req) {
          $links[] = $item['value']["field_{$item['type']}_name"][0]['value'];
        }
      } else {
        if (isset($item['value']) || isset($item['url']) || isset($item['fid'])) {
          if ($field != 'field_id_uri') {
            $links[] = $item['view'];
          } else {
            $links[] = l($item['view'], $item['view']);
          }
        } elseif (isset($item['safe']['nid'])) {
          $ids[] = $item['safe']['nid'];
          $links[] = l($item['safe']['title'], "node/{$item['safe']['nid']}");
          $expandable = true;
        }
      }
    }

    //flexifield without value for the selected language
    if (count($links) == 0) {
	  //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-793
	  //Remove 'To be completed' by an empty string when a fields is missing
      $vars[$field]['value'] = '';
      // Remove 'field_repository_schema' field when value is NULL
      if($field == 'field_repository_schema') {
        $key = array_search($field, $vars['fields']);
        unset($vars[$field]);
        unset($vars['fields'][$key]);
      }
      continue;
    }

    $ids = implode(',', $ids);
    $links = implode(', ', $links);
    $vars[$field]['value'] = $links;
    if ($expandable) {
      $vars[$field]['value'] .= "<div id='{$field}_button' style='display:inline;'>" .
        ctools_ajax_image_button(
          isa_ajax_get_image('collapsed'),
          "isa_ajax/nojs/teaser_expand/{$field}/{$ids}/lang={$lang_req}", t('expand')) .
        '</div>';
    }
  }
}

function _joinup_get_og_members($nid) {
  //rewrite the way you get the members for each
  //community

  $og_members = array(
    'Administrator(s)' => array(),
    'Facilitator(s)' => array(),
    'Member(s)' => array()
  );

  //currenty roles shown
  $og_members_roles = array (
    4 => 'Facilitator(s)',
    5 => 'Member(s)',
    6 => 'Administrator(s)');

  //loop through the roles
  foreach ($og_members_roles as $role => $role_name){
   // fecth users and users data from a community
   $result_users = db_query("SELECT * FROM {og_uid} ou
                  INNER JOIN {users} u ON ou.uid = u.uid
                  INNER JOIN {og_users_roles} ogr ON ogr.uid = u.uid
                  WHERE ou.nid = %d
                  AND u.status > 0
                  AND ou.is_active >= 1
                  AND ogr.gid = ou.nid
                  AND ogr.rid = %d
                  AND ou.is_admin >= 0
                  ORDER BY ou.created DESC LIMIT 3", $nid , $role);
     //loop though the results
      while ($user = db_fetch_object($result_users)) {
        //load profile from user
        $profile = content_profile_load('profile', $user->uid);
        //check if user is administrator/moderator for this comunity
        if ($user->is_admin) {
          $owner = l($profile->field_firstname[0]['value'] . ' ' . $profile->field_lastname[0]['value'], 'people/' . $user->uid);
          $roles = og_user_roles_get_roles_by_group($nid, $user->uid);
          /*
          if (!empty($roles)) {
            $owner .= '<div class="owner-roles">';
            foreach ($roles as $key => $value) {
              $name = db_result(db_query("SELECT name FROM role WHERE rid=%d", $key));
              $owner .= check_plain($name) . ' ';
            }
            $owner .= '</div>';
          }
           * 
           */
          $og_members['Administrator(s)'][$user->uid] = $owner;
        }
        //create the output variable of memebers list.
        $role = $role_name;
        $og_members[$role][$user->uid] = l($profile->field_firstname[0]['value'] . ' ' . $profile->field_lastname[0]['value'], 'people/' . $user->uid);
      }
    }
    return $og_members;
}

function joinup_clean_css_identifier($identifier, $filter = array(' ' => '-', '_' => '-', '/' => '-', '[' => '-', ']' => '')) {
  // By default, we filter using Drupal's coding standards.
  $identifier = strtr($identifier, $filter);

  // Valid characters in a CSS identifier are:
  // - the hyphen (U+002D)
  // - a-z (U+0030 - U+0039)
  // - A-Z (U+0041 - U+005A)
  // - the underscore (U+005F)
  // - 0-9 (U+0061 - U+007A)
  // - ISO 10646 characters U+00A1 and higher
  // We strip out any character not in the above list.
  $identifier = preg_replace('/[^\x{002D}\x{0030}-\x{0039}\x{0041}-\x{005A}\x{005F}\x{0061}-\x{007A}\x{00A1}-\x{FFFF}]/u', '', $identifier);

  return $identifier;
}