<?php

/*
 * Copyright (C) Atos
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

namespace Adapters;

define('CREATION_URI', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type');

/**
 * This class is an implementation of the interfaces ILoader and IExporter
 * @see util\Adapters\ILoader
 * @see util\Adapters\IExporter
 */
class XmlRdfAdapter implements ILoader, IExporter {

  /**
   * The logger
   * @var Logger
   */
  private $_logger;
  /**
   * This table is an ignore list for properties which are not neccessary to be
   * implemented
   * @var array
   */
  private $_ignore = array(
    // Comments removed from the ignore list for ISAICP-638
    //'http://www.w3.org/2000/01/rdf-schema#comment', // Comments
    'http://www.w3.org/2004/02/skos/core#conceptscheme', // Definition of the theme
    'http://purl.org/dc/terms/mediatypeorextent', // Media types
    );

  /**
   * Defined classes for ADMSx from ADMSx folder. In contains the classes
   */
  private $definedClasses = array();
  /**
   * Defined properties for ADMSx from ADMSx folder. In contains the properties
   */
  private $definedProperties = array();
  /**
  * ADMS processed objects
  */
  private $admsObjects = array();
  /**
   * This method is the constructor of the class
   * it uses the library for the implementation
   */
  public function __construct($debugLevel = '') {
    // Creating the log instance
    $this->_logger = \Logger::getLogger(__CLASS__);
    if ("" != $debugLevel) {
      $this->_logger->setLevel($debugLevel);
    }

    // Check that the variable is correctly configured
    if (!defined('RDFAPI_INCLUDE_DIR')) {
      throw new \InvalidArgumentException('Library RAP RDFAPI_INCLUDE_DIR not configured');
    }

    // Check that the path exists
    if (!is_dir(RDFAPI_INCLUDE_DIR)) {
      throw new \InvalidArgumentException('Library RAP path incorrect');
    }

    // Loading of the RDFAPI library
    //$this->_logger->debug('Loading of the RAP Library');
    require_once RDFAPI_INCLUDE_DIR . 'RdfAPI.php';
    require_once RDFAPI_INCLUDE_DIR . 'util/RdfUtil.php';
    $classesFolder = dirname(__FILE__)  . '/../ADMSx/RdfClasses';
    if ($handle = opendir($classesFolder)) {
      while (false !== ($entry = readdir($handle))) {
       if ($entry != "." && $entry != ".." && pathinfo($entry, PATHINFO_EXTENSION) == 'php') {
        $classname = str_replace(".php", "", "\\ADMSx\\RdfClasses\\" . $entry);
        $this->_logger->trace("ADMSx class definition: " . $classname);
        if (!class_exists ($classname)) {
          $this->_logger->debug("Class is not defined: " . $classname);
        } else {
          $this->_logger->trace("Class is defined: " . $classname);
        }
        $class = new $classname;
        $definedClasses = $class->getType();
        if (count($definedClasses) < 1) {
          $this->_logger->debug("Class: " . $classname . " doesn't have defined classes");
        }
        unset($class);
        foreach ($definedClasses as $key => $value) {
          $value = strtolower($value);
          $this->definedClasses[$value] = $classname;
          $this->_logger->trace("Url: " . $value . " => Class: " . $classname);
        }
      }
    }
    closedir($handle);
  } else {
    $this->_logger->error("Classes folder not found: " . $classesFolder);
    throw new \RuntimeException("Classes folder not found: " . $classesFolder);
  }
}

  /**
   * This method loads a table of AdmsObject from an RDF XML file
   * @param mixed $object the path to the file which should be loaded
   * @return obj a table of AdmsObject
   */
  public function load($object) {
    $this->admsObjects = array();
    // Check the input of the function
    // If it is a string, then it should be a valid file
    if (!is_string($object)) {
      throw new \InvalidArgumentException('Invalid parameter $object');
    }
    //variables declaration
    $intObjects = array();

    // Create a new MemModel
    $modelFactory = new \ModelFactory();
    $model = $modelFactory->getDefaultModel();
    // Load and parse document
    try {
      if (is_file($object))
        $model->load($object);
      else
        $model->loadFromString($object, 'rdf');
    } catch (\Exception $e) {
      $this->_logger->fatal($e->getMessage());
      return;
    }
    // Map the model to a table of AdmsObject
    $it = $model->getStatementIterator();
    //var_dump( $model );
    $pending_tab = array();
    while ($it->hasNext()) {
      $statement = $it->next();
      $this->_logger->trace("Subject: {$statement->getSubject()->toString()}");
      $this->_logger->trace("Predicate: " . $statement->getPredicate()->getURI());
      $this->_logger->trace("Object: {$statement->getObject()->toString()}");

      // if this is a type we are creating it
      // Get the subject
      $subject = $statement->getSubject();

      // In order to synchronize the various parameter and the different types
      // I am using the "intermediate" parameters of the Common\Object class
      // 1. Register all the parameters
      // 2. Register all the types to be instanciated
      // 3. Translate into an array of ADMS objects
      //
      if (!array_key_exists($subject->getURI(), $intObjects)) {
        // The object doesn't exist, we have to instantiate it!
        if ($statement->getPredicate()->getURI() == CREATION_URI) {
          $resource = $statement->getObject()->getURI();
          try {
            $type = $this->_isShortResource($resource) ?
            $this->_getClassNameFromResource($resource) :
            $this->_getClassFromURI($resource);
          } catch (\DomainException $de) {
            $this->_logger->debug($de->getMessage());
          } catch (\Exception $e) {
            $this->_logger->warn($e->getMessage());
            continue;
          }
          $object = new \Common\Object();
          $object->addType($type);
          $object->id = $subject->getURI();
          $intObjects[$subject->getURI()] = $object;
          //check if pending properties for this uri
          if (array_key_exists($subject->getURI(), $pending_tab)) {
            $counter = count($pending_tab[$subject->getURI()]);
            for ($i = 0; $i < $counter; $i++) {
              if (array_key_exists($i, $pending_tab[$subject->getURI()])) {
                if ($this->add_pending_property($intObjects, $pending_tab[$subject->getURI()][$i])) {
                  unset($pending_tab[$subject->getURI()][$i]);
                  $counter++;
                }
              }
            }
          }
        } else {
          // put the statement in pending
          $pending_tab[$subject->getURI()][] = $statement;
        }
      } else if ($statement->getPredicate()->getURI() == CREATION_URI) {
        // If it is the creation URI then, I just have to register a new
        // intermediate type and
        $resource = $statement->getObject()->getURI();

        try {
          $type = $this->_isShortResource($resource) ?
          $this->_getClassNameFromResource($resource) :
          $this->_getClassFromURI($resource);
          // I add the type
          $intObjects[$subject->getURI()]->addType($type);
        } catch (\DomainException $de) {
          $this->_logger->debug($de->getMessage());
        } catch (\Exception $e) {
          $this->_logger->warn($e->getMessage());
        }
      } else {
        // The Class already exists
        // We have to add the predicate with the object value
        $this->_logger->trace("Add a new predicate");
        $object = $intObjects[$subject->getURI()];
        $classname = $object->_types[0];
		
        try {
		  if(!$classname){
		    throw new \InvalidArgumentException(
			  "'{$subject->getURI()}' does not belong to a class recognized by the ADMS norm."
		    );
		  }
          // Check also, that the property exists for this object
          // Create a InvalidParameterException if doesn't exist
          $property = $this->_getPropertyFromURI(
            $statement->getPredicate()->getURI(),
            $classname);

          // $sob is the statement object which needs to be manipulated
          $sob = $statement->getObject();
          switch (get_class($sob)) {
            case 'Literal':
            $this->_logger->trace('The object is a literal');

              //I trim to avoid to much usage of <![CDATA[ xxx ]]>
            $objProperty = new \Common\LiteralProperty(
              trim($statement->getObject()->getLabel()),
              $sob->getLanguage(), $sob->getDatatype());
            break;
            case 'Resource':
            $objProperty = new \Common\Property(
              $statement->getObject()->getURI(),
              COMMON_PROPERTY_TYPE_RESOURCE
              );
            break;
            default:
            throw new \RuntimeException(
              'The type of statement object "' .
              get_class($sob) .
              '" is not implemented');
          }
		  foreach ($model->parsedNamespaces as $key=>$term){
		      if (strpos($statement->getPredicate()->getURI(),$key) !== false){
			    $orig_term=$term;
				$uri = explode("/",$statement->getPredicate()->getURI());
				$uri = end($uri);
				if(strpos($uri,"#") !== false){
				  $uri = explode("#", $uri);
				  $uri = end($uri);
				
				}
				$orig_term .= ":".$uri;
			  }
		    }
			if($orig_term){
			  $objProperty->additionalParameter['orig_term']=$orig_term;
			}
          $object->addParameter($property, $objProperty);
        } catch (\DomainException $de) {
          $this->_logger->debug(
            $de->getMessage() .
            " [Subject: {$statement->getSubject()->toString()}]");
        } catch (\Exception $e) {
          $this->_logger->error(
            $e->getMessage() .
            " [Subject: {$statement->getSubject()->toString()}]");
        }
      }
      end($intObjects);
      $intObjects ? $this->createAdmsObjects($intObjects[key($intObjects)], key($intObjects)) : FALSE;
    }
    return $this->admsObjects;
  }
  public function createAdmsObjects($intObject, $key) {
    $this->_logger->trace("Translation URI '{$URI}'");
    $this->admsObjects[$key] = $intObject->translate();
  }
  /**
   * This function needs to be implemented in order to export the data
   * @param adms:AdmsObject[] data the data (AdmsObject table) which should be exported
   * @param string output another output than the simple boolean value
   * @throws \DomainException
   * @throws \InvalidArgumentException
   * @return boolean
   * @access public
   */
  public function export($data, & $output, $import) {
    if (!is_array($data)) {
      throw new \InvalidArgumentException('The data sent to the export method are invalid');
    }

    // Initialisation of the RDF document
    $modelFactory = new \ModelFactory();
    $model = $modelFactory->getDefaultModel();

    $namespaces = \ADMSx\ADMSNamespaces::getNamespacesToExport();
    foreach ($namespaces as $key => $value) {
      $model->addNamespace($key, $value);
    }
    foreach ($data as $uri => $objects) {
      //For each objects, I create a new rdf subject
      foreach ($objects as $object) {
        try {
          // Create the creation statement regarding the URI
          $model->add(new \Statement(
            new \Resource($uri),
            new \Resource(CREATION_URI),
            new \Resource($this->_getURIFromClass(get_class($object), 'admsx'))
            ));
          if(method_exists($object, 'getProperties')) {
            $properties = $object->getProperties();
          }
          // We are searching for the all the properties
          foreach ($properties as $propertyKey => $propertyValue) {
            $name = $propertyKey;
            $values = $object->$name;
            // We are not treating the 'id' cause it is created above
            // If the value is not set, we don't treat it
            if ($name == 'id' || !isset($values))
              continue;

            $this->_logger->trace("Checking the values for the parameter {$name}");
            // If this property is registered as a resource we create a resource
            // otherwise a Literal object
            foreach ($values as $value) {
              if (!($value instanceof \Common\Property)) {
                throw new \DomainException(
                  "The property '{$name}' contains an invalid type of value");
              }
              $classname = get_class($object);
              $url = $this->_getURIFromProperty($name, $classname);
              if ($value->type == COMMON_PROPERTY_TYPE_RESOURCE) {
                if ('' != $url) {
                 $model->add(new \Statement(
                    new \Resource($uri),
                    new \Resource($url),
                    new \Resource($value->value)
                    )); 
                }
              } else {
                if (empty($value->value)) {
                  continue;
                }
                $literal = new \Literal($value->value);
                if (array_key_exists('language', $value->additionalParameter))
                  $literal->setLanguage($value->additionalParameter['language']);
                if (array_key_exists('datatype', $value->additionalParameter))
                  $literal->setDatatype($value->additionalParameter['datatype']);
                if ('' != $url) {
                  $model->add(new \Statement(
                    new \Resource($uri),
                    new \Resource($url),
                    $literal
                    ));
                }
              }
            }
          }
        } catch (\Exception $e) {
          $this->_logger->trace("Subject: {$e->getMessage()}");
        }
      }
    }

    $serializer = new \RdfSerializer();
    $serializer->configUseAttributes(false);
    $serializer->configUseAdvertisement(false);
	$defined_namespaces = $model->getParsedNamespaces();
	$default_namespaces = $serializer->m_defaultNamespaces;
	foreach($defined_namespaces as $key => $value){
	  if(isset($default_namespaces[$value])){
	    $default_namespaces[$value] = $key;
	  }
	}
	$serializer->m_defaultNamespaces = $default_namespaces;
    $output = $serializer->serialize($model);
    return true;
  }

  /**
   * This function is retrieving the property related to an URI
   * @param string $URI The URI which needs to be translated
   * @param classname
   * @return string
   * @throws InvalidArgumentException if the URI doesn't exist
   * @throws DomainException if the URI is ignored
   */
  private function _getPropertyFromURI($URI, $classname) {
    $object = new $classname;
    $properties = $object->getFlatProperties();
    unset($object);
    if (in_array(strtolower($URI), $this->_ignore)) {
      throw new \DomainException("The property '{$URI}' is ignored");
    }
    $properties = array_change_key_case($properties, CASE_LOWER);
    if (isset($properties[strtolower($URI)])) {
      return $properties[strtolower($URI)];
    }
    throw new \InvalidArgumentException(
      "The URI '{$URI}' is not a property of the ADMS norm."
      );
  }

  /**
   * This function is retrieving the class related to an URI
   * @param string $URI The URI which needs to be translated
   * @return string
   * @throws InvalidArgumentException if the URI doesn't exist
   * @throws DomainException if the URI is ignored
   */
  private function _getClassFromURI($URI) {
    $URI = strtolower($URI);
    if (in_array(strtolower($URI), $this->_ignore)) {
      throw new \DomainException("The class '{$URI}' is ignored");
    }
    if ($this->definedClasses[$URI]) {
      return $this->definedClasses[$URI];
    }
    throw new \InvalidArgumentException(
      "The URI '{$URI}' is not a class of the ADMS norm."
      );
  }

  /**
   * This function is retrieving the URI related to a property
   * @param string $property The property which needs to be translated
   * @param string $classname The name of the class when we want to extract a specific property
   * @return string
   * @throws InvalidArgumentException if the property is not referenced
   */
  private function _getURIFromProperty($property, $classname) {
    $namespaces = \ADMSx\ADMSNamespaces::getCombinedNamespaces();
    $object = new $classname;
    $properties = $object->getProperties();
    unset($object);
    $property = $properties[$property]['namespace']['admsx'];
    if (isset($property)) {
      $spliProperty = explode(":", $property);
      return $namespaces[$spliProperty[0]] . $spliProperty[1];
    }
    return '';
  }

  /**
   * This function is retrieving the URI related to a class
   * @param string $class The class which needs to be translated
   * @return string
   * @throws InvalidArgumentException if the property is not referenced
   */
  private function _getURIFromClass($class, $spec) {
    $class = new $class;
    if ($class->type) {
      if ('' != $spec) {
        return $class->type[$spec];
      }
    }
    return false;
  }

  /**
   * Get the classname of an object from its ressource definition
   * (e.g. adms:SemanticAssetRepository => \ADMS\SemanticAssetRepository )
   * @param string $ressource
   * @return string
   */
  private function _getClassNameFromResource($ressource) {
    $tclass = explode(':', $ressource);
    switch ($tclass[0]) {
      case 'adms':
      $tclass[0] = '\ADMS\\';
      break;
      case 'rad':
      $tclass[0] = '\RADion\\';
      default:
      break;
    }
    return implode('', $tclass);
  }

  /**
   * Get the ressource name of an object from its class name
   * (e.g. \ADMS\SemanticAssetRepository => adms:SemanticAssetRepository )
   * @param string $classname
   * @return string
   */
  private function _getResourceFromClassName($classname) {
    $tclass = explode('\\', $classname);
    switch ($tclass[0]) {
      case 'ADMS':
      $tclass[0] = 'adms:';
      break;
      case 'RADion':
      $tclass[0] = 'rad:';
      default:
      break;
    }
    return implode('', $tclass);
  }

  /**
   * This method is used to check if the string passed is a resource
   * @param string $resource
   */
  private function _isShortResource($resource) {
    return preg_match('/^[A-Za-z]*:[A-Za-z]*$/', $resource);
  }

//end of load() function

  /**
   * This method is used to add to adms Object, property in pending
   * when declared before the type of object
   * @param string $resource
   */
  private function add_pending_property(&$intObjects, $statement) {
    // The Class already exists
    // We have to add the predicate with the object value
    $subject = $statement->getSubject();
    $this->_logger->trace("Add a new predicate");
    $object = $intObjects[$subject->getURI()];
    $classname = $object->_types[0];
    try {
      // Check also, that the property exists for this object
      // Create a InvalidParameterException if doesn't exist
      $property = $this->_getPropertyFromURI(
        $statement->getPredicate()->getURI(),
        $classname
        );

      // $sob is the statement object which needs to be manipulated
      $sob = $statement->getObject();

      switch (get_class($sob)) {
        case 'Literal':
        $this->_logger->trace('The object is a literal');

          //I trim to avoid to much usage of <![CDATA[ xxx ]]>
        $objProperty = new \Common\LiteralProperty(
          trim($statement->getObject()->getLabel()),
          $sob->getLanguage(), $sob->getDatatype());
        break;
        case 'Resource':
        $objProperty = new \Common\Property(
          $statement->getObject()->getURI(),
          COMMON_PROPERTY_TYPE_RESOURCE
          );
        break;
        default:
        throw new \RuntimeException(
          'The type of statement object "' .
          get_class($sob) .
          '" is not implemented');
      }
      $object->addParameter($property, $objProperty);
    } catch (\DomainException $de) {
      $this->_logger->debug(
        $de->getMessage() .
        " [Subject: {$statement->getSubject()->toString()}]");
    } catch (\Exception $e) {
      $this->_logger->error(
        $e->getMessage() .
        " [Subject: {$statement->getSubject()->toString()}]");
    }

    return true;
  }

}

// end of XmlRdfLoade