<?php

namespace Common;

class ADMSObject {

  /**
   * Node converted from ADMS Object
   * @var \stdClass
   * @access protected
   */
  public $node;

  /**
   * Pointer to the array of all ADMS Objects
   * @var array
   * @access public
   */
  public $data;

  /**
   * Pointer to the task array
   * @var array
   * @access public
   */
  public $output;

  /**
   * Content type of the ADMS Object
   * @var string
   * @access protected
   */
  public $content_type;

  /**
   * Object with validation methods
   * @var Validator
   * @access protected
   * @see __construct()
   */
  public $_validator;

  /**
   * Contains the defined types of the asset repository that could exist
   * @var array
   * @access protected
   */
  public $type = array();

  /**
   * Contains the properties of the Asset Repository
   * @access protected
   * @va array
   */
  public $properties = array();

  /**
   * Check if the current object is already imported
   * @var bool
   */
  public $imported = FALSE;

  /**
   * Returns a Logger tool
   * @return \Logger
   */
  public function getLogger() {
    return $this->_logger;
  }

  /**
   * Object with methods for logging
   * @var \Logger
   */
  protected $_logger;

  /**
   * @var string
   */
  public $item_class;

  /**
   * @var string
   */
  public $uri;

  public $error_message;
  public $tStatus;

  /**
   * Bootstrap the ADMSObject
   */
  function __construct() {
    $this->item_class = get_class($this);
    $this->node = new \stdClass();
    $this->_logger = \Logger::getLogger(__CLASS__);
    $this->_validator = new Validator($this);
    $this->_importer = new Importer($this);
  }

  public function getIdFromNode($node){
    return isset($node->field_id_uri[0]['value'])
      ? $node->field_id_uri[0]['value']
      : url($node->path, array('absolute' => TRUE));
  }

  public function toADMSObjectGeneral($node, $properties) {
    $transformFunctions = $this->commonFunctions();

    $this->id = $this->getIdFromNode($node);

    foreach ($properties as $key => $property) {
      if ("" != $property['map'] && (
          "" != $property['node_field']
          || "" != $property['extra_language']
          || "" != $property['taxonomy_field']
          || "" != $property['organic_group_field'])
      ) {
        $value = $this->getPropertyValues($node, $property);
        $functionTransform = $transformFunctions[$property['map']];
        //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1168
        //Avoid export empty properties
        $propertyValue = $functionTransform($value);
        if (count($propertyValue) > 0) {
          $this->{$key} = $propertyValue;
        }
      }
    }
  }

  protected function commonFunctions() {
    $transformations['literal'] = function ($value, $datatype) {
      $objects = array();
      foreach ($value as $key => $subValue) {
        //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1168
        //Avoid export empty properties
        if ($subValue['value'] != '') {
          $objects[] = new \Common\LiteralProperty($subValue['value'], $subValue['language'], $subValue['datatype']);
        }
      }
      return $objects;
    };
    $transformations['property'] = function ($value, $type = COMMON_PROPERTY_TYPE_RESOURCE, $additionalParameter = array()) {
      $objects = array();
      foreach ($value as $key => $subValue) {
        //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1168
        //Avoid export empty properties
        if ($subValue['value'] != '') {
          $objects[] = new \Common\Property($subValue['value'], $type, $additionalParameter);
        }
      }
      return $objects;
    };
    return $transformations;
  }

  protected function getPropertyValues($node, $property) {
    $propertyValues = array();
    $cardinality = $property['cardinality'][1];
    if (TRUE == $property['organic_group_field']) {
      if($node->type=="repository"){
       $propertyValues = $this->getOrganicGroupChildren($node, $property);
      }else{
       $propertyValues[] = $this->getOrganicGroupParent($node, $property);
      }
    }
    else {
      if ("" != $property['taxonomy_field']) {
        $term = $this->getTaxonomyValues($node, $property);
        if ($term) {
          $propertyValues = $term;
        }
      }
      else {
       if($property['value_key']=='taxonomy'){
        // $propertyValues[] = array("value" => $this->getLocaleLanguage($node, $property), "language" => {$property['node_field']}[0]['value']});
        $propertyValues[] = $this->getLocaleLanguage($node, $property);
       }else{
        if ($property['node_field']) {
          $value[$property['node_field']] = is_array($node->$property['node_field']) ? $node->$property['node_field'] : array($node->$property['node_field']);
        }
        if (count($property['extra_language']) > 0) {
          foreach ($property['extra_language'] as $key => $extraValue) {
            $value[$extraValue] = is_array($node->$extraValue) ? $node->$extraValue : array($node->$extraValue);
          }
        }

        foreach ($value as $field => $fieldValue) {
          foreach ($fieldValue as $key => $subValue) {
            $result = $this->getValue($field, $subValue, $property);
            if (count($result) > 0) {
              $propertyValues[] = $this->getValue($field, $subValue, $property);
            }
          }
        }
       }
      }
    }
    if (1 == $cardinality) { //Only accepts up to one value
      $propertyValues = array($propertyValues[0]);
    }

    return $propertyValues;
  }

  protected function getValue($field, $value, $property) {
    if ('field_asset_node_reference' == $field) {
      $node = node_load($value['value']['field_asset_node_reference_node'][0]['nid']);
      $extra_key = $property['extra_switch'][key($property['extra_switch'])];
      if (is_object($node) and $value['value']['field_asset_node_relationship'][0]['value'] == $extra_key) {
        $propertyValues['value'] = $node->field_id_uri[0]['value'];
        $propertyValues['language'] = "en";
        return $propertyValues;
      }
    }
    if (count($property['extra_language']) > 0 and in_array($field, $property['extra_language'])) {
      if ('language_textarea' == $value['type']) {
        $propertyValues['value'] = $value['value']['field_language_textarea_name'][0]['value'];
        $propertyValues['language'] = $value['value']['field_language_textfield_lang'][0]['value'];
      }
      if ('language_textfield' == $value['type']) {
        $propertyValues['value'] = $value['value']['field_language_textfield_name'][0]['value'];
        $propertyValues['language'] = $value['value']['field_language_textfield_lang'][0]['value'];
      }
      return $propertyValues;
    }
    if ($field == $property['node_field']) {
      if ('nid' == $property['value_key']) {
        $node = node_load($value['nid']);
        if (isset($property['extra_switch'])) {
          $extra_key = key($property['extra_switch']);
          if ($property['extra_switch'][$extra_key] == $node->{$extra_key}[0]['value']) {
            $propertyValues['value'] = $node->field_id_uri[0]['value'];
            $propertyValues['language'] = 'en';
          }
        }
        else {
          $propertyValues['value'] = $node->field_id_uri[0]['value'];
          $propertyValues['language'] = 'en';
        }
      }
      else {
        if ($property['value_key']) {
          if ('created' == $property['value_key'] or 'modified' == $property['value_key']) {
            $date = date('c', $value);
            $propertyValues['value'] = $date;
            $propertyValues['datatype'] = "http://www.w3.org/2001/XMLSchema#dateTime";
          }
          else {
            $propertyValues['value'] = $value[$property['value_key']];
            $propertyValues['language'] = 'en';
          }
        }
        else {
          $propertyValues['value'] = $value;
          $propertyValues['language'] = 'en';
        }
      }
    }
    return $propertyValues;
  }

  protected function getTaxonomyValues($node, $property) {
    foreach ($node->taxonomy as $key => $value) {
      if ($property['taxonomy_field'] == $value->vid) {
        $fields = term_fields_get_fields_values($value);
        $term['value'] = $fields['taxonomy_uri_value'];
        $term['language'] = 'en';
        $propertyValues[] = $term;
      }
    }
    return $propertyValues;
  }

  protected function getLocaleLanguage($node, $property) {
   $language_code = $node->{$property['node_field']}[0]['value'];
   $domain = db_result(db_query("SELECT domain FROM {isa_language_legacy} WHERE language = '%s'", $language_code));

   return array('value' => $domain, 'language' => $language_code);
  }

  protected function getOrganicGroupParent($node, $property) {
    $group = og_get_node_groups($node);
    foreach ($group as $key => $value) {
      $type = db_result(db_query("SELECT type FROM node WHERE nid=%d AND type='repository'", $key));
      if ('' != $type) {
        $parent = node_load($key);
        return array(
          'value' => $parent->field_id_uri[0]['value'],
          'language' => 'en'
        );
      }
    }
  }

  protected function getOrganicGroupChildren($node, $property) {
    $sql = "SELECT oga.nid as nid, n.title FROM {node} n INNER JOIN {og_ancestry} oga ON n.nid = oga.nid WHERE oga.group_nid = %d";
    $result = db_query($sql, $node->nid);

    while ($row = db_fetch_object($result)) {
     $groups[$row->nid] = $row->title;
    }

    $group_id = array();
    foreach ($groups as $key => $value) {
      $parent = node_load($key);
      $group_id[] = array(
        'value' => $parent->field_id_uri[0]['value'],
        'language' => 'en'
      );
    }

    return $group_id;
  }

  public function getADMSxClassUri() {
    return $this->type['admsx'];
  }

  public function getProperties() {
    return $this->properties;
  }

  public function getFlatProperties() {
    $namespaces = \ADMSx\ADMSNamespaces::getCombinedNamespaces();

    foreach ($this->properties as $key => $value) {
      foreach ($value['namespace'] as $sub_key => $uri) {
        $splitUris = explode(":", $uri);
        $propertiesUris[$namespaces[$splitUris[0]] . $splitUris[1]] = $key;
      }
    }

    return $propertiesUris;
  }

  public function toADMSObject($node) {
    $this->toADMSObjectGeneral($node, $this->properties);
  }

  public function getType() {
    return $this->type;
  }

  public function getContentType() {
    return $this->content_type;
  }

  /**
   * Execute methods that loads a content into the node and check for some errors
   * @param $methods
   * @return bool
   */
  protected function executeLoadMethods($methods,$import=true) {
    foreach ($methods as $method) {
      if (preg_match('/^import/', $method)) {
        if (!method_exists($this, $method)) {
          $this->_importer->{$method}($import);
        }
        else {
          $this->{$method}();
        }
      }
      elseif (preg_match('/^check/', $method)) {
        $this->_validator->{$method}();
        if ($this->tStatus == 'error') {
          return FALSE;
        }
      }
    }
    return TRUE;
  }

  /**
   * Converts ADMSObject into drupal object
   */
  public function importToDrupal(&$data, &$output,$import=true) {
    // Link the data as reference to the current object
    $this->data = & $data;
    $this->output = & $output;
    // Check if the current object is a part of another (e.g. nodereference) and is already imported
    if (!$this->imported) {
      $this->imported = TRUE;
      $this->uri = trim($this->id, ISA_URI_TRIM_CHARS);
      $load_methods = array(
        'checkURI', // Checks URI
        'importNode', // Init an empty node object or loads an existant
        'checkBlacklist', // Check if node or content type are in black list
        'importEntitySpecificParameters', // Executes on the entity specific imports
        'importProperties', // Convert all adms object properties to the node
        'importLanguage', // Set node language
        'checkBodyField', // Check english body
        'checkTitleField', // Check english title
        'checkAccessURL', // check accessUrl
        'checkCardinality', // Check the properties cardinality vs converted to the node from adms object cardinality
        'importUri', // Associate id to node uri
      );
      if($import){
        $load_methods[]='importSaveNode';
        $load_methods[]='importWorkflow';
      }else{
       // $load_methods[]='importValidateNode';
      }
      $this->executeLoadMethods($load_methods,$import);
    }
  }
  /**
   * After all imports assign og groups ancestries
   * @param $property_name
   */
  public function configureOgAncestry() {
    foreach ($this->properties as $name => $property) {
      if (TRUE === $property['organic_group_field']) {//Operations for content related to og
        $objects = $this->$name;
        $this->node->og_groups = array();
        foreach($objects as $object) {
          $og_ancestry = isa_toolbox_get_node_id_by_uri($object->value, 'repository');
          if (!isset($og_ancestry)) {
            $og_ancestry = isa_toolbox_get_node_id_by_uri($object->value, 'project');
          }
          if ($og_ancestry) {
            $this->node->og_groups[$og_ancestry] = $og_ancestry;
            node_save($this->node);
          }

        }
      }
    }
  }
  /**
   * Get nodes property
   * @param $property_name
   */
  public function getNodeProperty($property) {
    if (is_array($property)) {
      if(isset($property['node_field'])) {
        $property = $property['node_field'];
      }elseif (isset($property['extra_language'])) {
        $property = $property['extra_language'][0];
      }elseif (isset($property['taxonomy_field'])) {
        $taxonomies = array();
        foreach ($this->node->taxonomy as $key => $value) {
         if ($property['taxonomy_field'] == $value->vid) {
          $taxonomies[] = $value;
         }
        }

        return $taxonomies;
      }
    }
    if (isset($this->node->{$property})) {
      return $this->node->{$property};
    }
  }
  protected function importEntitySpecificParameters(){
  }
}
