<?php

namespace Common;


class Importer {

  /**
   * Reference to the ADMS Object
   * @var ADMSObject
   */
  public $adms_object;

  /**
   * Valid dates for properties value_key
   * date > cck field, modified > node changed, created > node created
   * @var array
   */
  public $validDatesType = array('date','created','modified');

  function __construct(ADMSObject &$adms_object) {
    $this->adms_object = &$adms_object;
  }

  /**
   * Map a language from ADMSObject to the node
   * @return bool|string
   */
  public function importLanguage(){
    // Set the language
    if (isset($this->adms_object->language[0]->value)) {
      $code_lang = isa_toolbox_get_lang_code_byuri($this->adms_object->language[0]->value);
      $language = $code_lang ? $code_lang : 'en';
    } else {
      $language = "en";
    }
    $this->adms_object->node->language = $language;
  }

  /**
   * @return bool|string
   */
  public function importWorkflow(){
    $is_moderator = false;
    $node = &$this->adms_object->node;
    $type = $this->adms_object->getNodeProperty('type');

    if (!empty($this->adms_object->output['repository_id'])) {
      $node_og_group = node_load($this->adms_object->output['repository_id']);
      if ($node_og_group->type == ISA_REPOSITORY_TYPE) {
        if ($node_og_group->field_repository_harvestmethod[0]['value'] == REPOSITORY_HARVEST_MODE_AUTOMATIC) {
          $user_uid =  $node_og_group->uid ;
        } else {
          $user_uid = $node->revision_uid != '' ? $node->revision_uid :  $node->uid;
        }

        $node_user = user_load($user_uid);

        $moderator_rid = variable_get('moderator_rid', null);
        if ($moderator_rid && isset($node_user->roles[$moderator_rid])) {
          $is_moderator = true;
        }
      }
    }

    $workflow_state = null;
//set the workflow
    switch ($type) {
      case 'asset_release':
      $workflow_state = $is_moderator == true  ?  ISA_SID_ASSET_VALIDATED : ISA_SID_ASSET_PROPOSED;
      $creation_work = ISA_SID_ASSET_CREATION;
      break;
      case 'publisher':
      case 'licence':
// The licence and publisher uses the community workflow
      $workflow_state = $is_moderator == true  ?  ISA_SID_COMMUNITY_VALIDATED : ISA_SID_COMMUNITY_PROPOSED;
      $creation_work = ISA_SID_COMMUNITY_CREATION;
      break;
      case 'repository':
      $workflow_state = $is_moderator == true  ?  ISA_SID_REPOSITORY_VALIDATED : ISA_SID_REPOSITORY_PROPOSED;
      $creation_work = ISA_SID_REPOSITORY_VALIDATED;
      break;
      default:
// No workflow
      break;
    }
    if($workflow_state) {
      //If estate changes
      if ($workflow_state != $node->_workflow) {
        workflow_execute_transition($node, $workflow_state, 'Created by ADMS import', true);
      }
    }
  }

  /**
   * Load node by URI  or ID
   * @return bool|null|object|\stdClass
   */
  public function importNode(){
    //check if the node exist in drupal
    $node_nid = isa_toolbox_get_node_id_by_uri($this->adms_object->uri, $this->adms_object->content_type);
    //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-736
    //Check if there is a previously created node with a non trimmed url
    if(false == $node_nid){
      $node_nid = isa_toolbox_get_node_id_by_uri($this->adms_object->id, $this->adms_object->content_type);
    }

    if (is_numeric($node_nid)) {
      //load existing node
      $node = node_load($node_nid);
      //to allow revision
      //$node->revision_moderation=true;
      //$node->is_pending=true;
      if (isset($node->nid)) {
        $node->revision = 1;
        //check the state of the revision (draft or published)
        $node->revision_moderation = false;
        // JIRA ISAICP-707, remove moderation on the revision
        //$node->revision_moderation = $this->adms_object->_check_revision_state($node);
      }
    } else {
      //create a new node
      $node = new \stdClass();
      $node->type = $this->adms_object->content_type;
      $node->uid = $this->adms_object->output['user_id'];
    }
    $this->adms_object->node = $node;
  }
  public function importUri() {
    $this->adms_object->node->field_id_uri[0]['value'] = $this->adms_object->id;
  }

  public function importUrl($name, $property, $object) {
    $field_name = trim($property['node_field']);

    if(is_string($object)){
     $value = $object;
    } else {
     $value = $object->value;
    }

    if($this->adms_object->_validator->verifURL($value)){
     $this->importValue($property, $object);
    }else{
	 $error = "The {$this->adms_object->node->type} identifies an element for which the value of the {$name} property '{$value}' does not respect the URL syntax specified in RFC 1738.
     The element is ignored.";
	 $error = preg_replace('/(^| )a ([aeiouAEIOU])/', '$1an $2', $error);
     $this->adms_object->getLogger()->error($error);
    }
  }

  /**
   * Method for map the properties to the node
   */
  public function importProperties($import = TRUE) {
    $extra_switch_removed = FALSE;
    foreach ($this->adms_object->properties as $name => $property) {
      //Delete previous taxonomy values
      if ($property['taxonomy_field']) {
        foreach ($this->adms_object->node->taxonomy as $key => $value) {
          if ($property['taxonomy_field'] == $value->vid) {
            unset($this->adms_object->node->taxonomy[$key]);
          }
        }
      }

      //Property does not exist in RDF file
      if (!isset($this->adms_object->$name)) {
        //Delete value if it does not exist in the file
        if (isset($property['node_field']) && $property['node_field'] != "") {
          unset($this->adms_object->node->{$property['node_field']});
          if (isset($property['extra_switch']) && !$extra_switch_removed) {
            $extra_switch_removed = TRUE;
          }
        }

        //Delete Flexi field values
        if (isset($property['extra_language'])) {
          foreach ($property['extra_language'] as $value) {
            unset($this->adms_object->node->{$value});
          }
        }
        continue;
      }

      //Property do existi in RDF file
      $objects = $this->adms_object->$name;

      if (!isset($property['taxonomy_field'])) {
        if (TRUE === $property['organic_group_field']) { //Operations for content related to og
        }
        else {
          if (!isset($property['extra_language'])) {
            if (isset($property['extra_switch'])) {
              //Delete removed versions
              if (!$extra_switch_removed) {
                foreach ($this->adms_object->node->{$property['node_field']} as $key => $value) {
                  unset($this->adms_object->node->{$property['node_field']});
                }
                $extra_switch_removed = TRUE;
              }
            }
            else {
              $this->adms_object->node->{$property['node_field']} = array();
            }
          }
          else {
            foreach ($property['extra_language'] as $value) {
              unset($this->adms_object->node->{$value});
            }

            if (isset($property['node_field']) && $property['node_field'] != "") {
              unset($this->adms_object->node->{$property['node_field']});
            }
          }
        }
      }
      foreach ($objects as $object) {
        if (TRUE === $property['organic_group_field']) { //Operations for content related to og
          $og_ancestry = isa_toolbox_get_node_id_by_uri($object->value);
        }
        else {
          if (isset($property['extra_language'])) {
            if ('' == $property['node_field']) {
              $this->importFlexiField($property, $object);
            }
            else {
              //Import in the default language, english
              if ('en' == $object->additionalParameter['language']) {
                $this->importValue($property, $object);
              }
              else {
                $this->importFlexiField($property, $object);
              }
            }
          }
          else {
            if (isset($property['node_field'])) {
              if ($property['value_key'] == 'nid' || $property['value_key'] == 'item_id') {
                if (isset($property['extra_switch'])) {
                  $this->importFlexifieldExtraSwitch($property, $object, $import);
                }
                else {
                  $this->importNodeReference($property, $object, $import);
                }
              }
              elseif (in_array($property['value_key'], $this->validDatesType)) {
                $this->importDate($property, $object);
              }
              elseif ($property['value_key'] == 'url') {
                $this->importUrl($name, $property, $object);
              }
              elseif ($property['value_key'] == 'taxonomy') {
                $this->importLocaleLanguage($property, $object);
              }
              else {
                $this->importValue($property, $object);
              }
            }
            elseif (isset($property['taxonomy_field'])) {
              $this->importTaxonomy($name, $property, $object);
            }
          }
        }
      }
    }
  }

  /**
   * Method for map the property of taxonomy type to the node
   * @var $property
   * @var $object
   * @return bool
   */
  public function importTaxonomy($name, $property, $object) {
    $value = trim($object->value);
    $vid = $property['taxonomy_field'];
    $literal = $property['taxonomy_literal'];
    $node_field = $property['taxonomy_node_field'];

    if (!$vid || empty($value)) {
      return FALSE;
    }

    if (isset($literal) && $literal) {
      $tid = taxonomy_get_term_by_name($value);
      if (count($tid) > 0) {
        foreach ($tid as $key => $taxonomy) {
          if ($taxonomy->vid == $vid) {
            $tid = $taxonomy->tid;
          }
        }
      }
      //if no term found and taxonomy_create property is set to true create the term in the vid
      else if (isset($property['taxonomy_create']) && $property['taxonomy_create']) {
        $term = array(
          'name' => $value,
          'vid' => $vid,
        );
        if (taxonomy_save_term($term)) {
          //if term was succesfully saved set the tid value that will be used later in the function
          $tid = $term['tid'];
        }
      }
    }
    else {
      $tid = isa_toolbox_get_tid_byuri($value, $vid);
    }
    if (!$tid) {
      if (isset($object->additionalParameter['orig_term']) && !empty($object->additionalParameter['orig_term'])) {
        $type_term = $object->additionalParameter['orig_term'];
      }
      else {
        $type_term = $property['namespace']['admsx'];
      }

      $error = "'{$this->adms_object->id}' identifies a {$this->adms_object->node->type} that is declared to be of type {$type_term} which is not a term in the recommended controlled vocabulary. value: {$value} not in vid: " . $vid . " literal: " . $literal;
      $error = preg_replace('/(^| )a ([aeiouAEIOU])/', '$1an $2', $error);
      $this->adms_object->getLogger()->warn($error);
    }
    else {
      if (isset($node_field)) {
        $this->adms_object->node->{$node_field}[] = array('value' => $tid);
      }
      else {
        $term = taxonomy_get_term($tid);
        $this->adms_object->node->taxonomy[$tid] = $term;
      }
    }
  }

  /**
   * Validate nodes
   */
  public function importValidateNode(){
    // Regenerate the teaser
    $node = $this->adms_object->node;
    $node->teaser = node_teaser($node->body);
    $isnew = empty($node->nid);
    form_set_error(NULL, '', TRUE);
    node_validate($node);
    $errors = form_get_errors();
    $this->adms_object->getLogger()->info('The node '.$node->title. ' has the following errors:');
    $this->adms_object->getLogger()->info($errors);
  }

  /**
   * Save node operations
   */
  public function importSaveNode(){
    // Regenerate the teaser
    $node = &$this->adms_object->node;
    $node->teaser = node_teaser($node->body);
    $isnew = empty($node->nid);
    node_save($node);
    $this->adms_object->getLogger()->info("The object {$node->type} with the URI '{$this->adms_object->uri}' has been saved");
    return;
  }
  /**
   * Method for map the date property to the node
   * @var $property
   * @var $object
   * @return bool
   */
  public function importDate($property, $object){
    //date times [date > cck field, modified > node changed, created > node created]
    $value = $object->value;
    $field_name = trim($property['node_field']);

    //$date = new DateTime($value);
    $date = new \DateTime();
    $date = date_create($value);
    $date_errors = date_get_last_errors();

    if (($date_errors['warning_count'] == 0) && ($date_errors['error_count'] == 0)) {
      switch ($property['value_key']) {
        case 'created':
        case 'modified':
        $this->adms_object->node->{$field_name}= $date->format('U');
        break;
        case 'date':
        $this->adms_object->node->{$field_name}[]['value']  = $date->format('c');
        break;
      }
    }
    else {
		$error = "'{$this->adms_object->uri}' identifies an element for which the value '{$value}' of a date-related property is not appropriately typed. '{$value}' is not an ISO 8601 (date/time) datatype.";
		$error = preg_replace('/(^| )a ([aeiouAEIOU])/', '$1an $2', $error);
		$this->adms_object->getLogger()->warn($error);
    }
  }

  /**
   * Method for map dct:language to Locale Languages
   * @var $property
   * @var $object
   * @return bool
   */
  public function importLocaleLanguage($property, $object){
   $value = $object->value;
   $field_name = trim($property['node_field']);

   $language = db_fetch_object(db_query("SELECT * FROM {isa_language_legacy} WHERE domain = '%s'", $value));
   if (isset($language) && $language!="") {
    $this->adms_object->node->{$field_name}[]['value']= $language->language;
   }else{
    if(isset($object->additionalParameter['orig_term'])&&!empty($object->additionalParameter['orig_term'])){
     $type_term = $object->additionalParameter['orig_term'];
    }else{
     $type_term = $property['namespace']['admsx'];
    }
    $error = "'{$this->adms_object->id}' identifies a {$this->adms_object->node->type} that is declared to be of type {$type_term} which is not a term in the recommended controlled vocabulary.";
    $error = preg_replace('/(^| )a ([aeiouAEIOU])/', '$1an $2', $error);
    $this->adms_object->getLogger()->warn($error);
   }
  }

    /**
   * Import for flexifields with selectors in their value
   * @var $property
   * @var $object
   */
    public function importFlexifieldExtraSwitch($property, $object,$import=true) {
      if ('asset_node_reference' == $property['extra_switch_type']) {
        $field_name = $property['node_field'];
        $extraSwitch = key($property['extra_switch']);
        $reference = isa_toolbox_get_node_id_by_uri($object->value, 'asset_release');

        if(is_numeric($property['extra_switch'][$extraSwitch])){
         $field = array(
           'type' => 'asset_node_reference',
           'value' => array(
             'field_asset_node_reference_node' => array(array('nid' => $reference)),
             $extraSwitch => array(array('value' => $property['extra_switch'][$extraSwitch]))
             )
           );

          array_push($this->adms_object->node->{$field_name}, $field);
        }else{
         //Checks if the field already exits for that type
         if(count($this->adms_object->node->{$extraSwitch})==0 || empty($this->adms_object->node->{$extraSwitch}[0]['nid'])){
          $field = array(
            'nid' => $reference
          );

          $this->adms_object->node->{$extraSwitch}[0] = $field;
         }else{
          $this->adms_object->getLogger()->warn("{$this->adms_object->node->type} has more than one current version attributes associated.
          The system will save only the first one.");
         }
        }
      }
      elseif('agent_node_reference' == $property['extra_switch_type']) {
        $field_name = $property['node_field']; //field_sw_project_agents
        $extraSwitch = key($property['extra_switch']);  //'field_sw_project_agent_type' => 'contributor'

        $reference = isa_toolbox_get_node_id_by_uri($object->value, 'software_project_agent');
        if ($reference) {
         $data_object = &$this->adms_object->data[$object->value][0];
         $data_object->node = node_load($reference);
         $data_object->importToDrupal($this->adms_object->data, $this->adms_object->output,$import);

         $nid = $data_object->node->nid;

         $node_agent_type = $data_object->node->field_sw_project_agent_type;

         //Validate that the type is not duplicate
         $insert_type = TRUE;
         foreach ($node_agent_type as $key => $value) {
           if($value['value']==$property['extra_switch'][$extraSwitch]){
             $insert_type = FALSE;
           }
         }

         if($insert_type){
           $data_object->node->{$extraSwitch}[] = array('value' => $property['extra_switch'][$extraSwitch]);
           node_save($data_object->node);
         }

         $insert_node = TRUE;
         foreach ($this->adms_object->node->{$field_name} as $key => $value) {
           if($value['nid']==$nid){
             $insert_node = FALSE;
           }
         }
         if($insert_node){
           $field = array(
             'nid' => $nid
           );
           $this->adms_object->node->{$field_name}[] = $field;
         }
        }
      }
    }
  /**
   * Method for map the property of Node Reference type to the node
   * @var $property
   * @var $object
   * @return bool
   */
  public function importNodeReference($property, $object,$import=true){
    $value = $object->value;
    $field_name = trim($property['node_field']);
    $field_value_key = trim($property['value_key']);
    $nid = null;
    if(!$field_name) return false;
    if(isset($this->adms_object->data[$value][0])){
      /** @var  $data_object \Common\ADMSObject */
      $data_object = &$this->adms_object->data[$value][0];
      $nid = isa_toolbox_get_node_id_by_uri($value, $data_object->content_type);
      if($nid){
        $data_object->node = node_load($nid);
      }

      if($field_name=='periodOfTime'){
       //Get PeriodOfTime as fields
       $data_period = $this->adms_object->data[$value];
       if(isset($data_period)){
        $start_date = $data_period[0]->startDate;
        $end_date = $data_period[0]->endDate;

        $count_node = count($this->adms_object->node->{$data_period[0]->properties['startDate']['node_field']});

        $this->adms_object->node->{$data_period[0]->properties['startDate']['node_field']}[$count_node][$data_period[0]->properties['startDate']['value_key']] = $start_date[0]->value;
        $this->adms_object->node->{$data_period[0]->properties['endDate']['node_field']}[$count_node][$data_period[0]->properties['endDate']['value_key']] = $end_date[0]->value;
       }
      }else{
       // Get node by uri
       $data_object->importToDrupal($this->adms_object->data, $this->adms_object->output,$import);
       $nid = $data_object->node->nid;

       if($field_name=='field_asset_identifier'){
         //Get Identifier as fields
         $this->adms_object->node->{$field_name}[] = array(
            'type' => 'identifier',
            'value' => array(
              'field_identifier_content' => array(array('value' => $data_object->node->field_identifier_content[0]['value'])),
              'field_identifier_scheme' => array(array('value' => $data_object->node->field_identifier_scheme[0]['value'])),
              'field_identifier_scheme_version' => array(array('value' => $data_object->node->field_identifier_scheme_version[0]['value'])),
              'field_identifier_scheme_agency' => array(array('value' => $data_object->node->field_identifier_scheme_agency[0]['value'])),
            ),
            'item_id' =>$nid
          );
       }else{
        $this->adms_object->node->{$field_name}[] = array($field_value_key => $nid);
       }
      }
    }
    else {
      return false;
    }
  }
    /**
   * Method for map the property of Flexi field type to the node
   * @var $property
   * @var $object
   * @return bool
   */
    public function importFlexiField($property, $object){
      $value = $object->value;
      $field_name = trim($property['extra_language'][0]);
      $node_field = trim($property['node_field']);
      $type = $this->adms_object->node->type;
      $nid = null;
      if(!$field_name) return false;
      $field = $this->adms_object->node->{$field_name};
      //Get field names
      if ('' == $field[0]['type']) {
        $field = array();
      }
      switch ($property['extra_language_type']) {
        case 'language_textarea':
        $language_field = 'field_language_textfield_lang';
        $language_value = 'field_language_textarea_name';
        break;
        case 'language_textfield':
        $language_field = 'field_language_textfield_lang';
        $language_value = 'field_language_textfield_name';
        break;
        default:
        return false;
        break;
      }
      if(isset($object->additionalParameter['orig_term'])&&!empty($object->additionalParameter['orig_term'])){
       $type_term = $object->additionalParameter['orig_term'];
      }else{
       $type_term = $property['namespace']['admsx'];
      }

      $addField = TRUE;
      //Verify description have more than 5 words
      //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1168
      //Description must have at least one word
      if(($type=='asset_release' || $type=='repository')
      && $property['node_field']=='body' && str_word_count($object->value)<1){
	     $error = "{$this->adms_object->id} identifies a {$type_term} that has this description '{$object->value}'. A description should contain at least 1 words.
       Please update the description and load the data again";
	     $error = preg_replace('/(^| )a ([aeiouAEIOU])/', '$1an $2', $error);
       $this->adms_object->getLogger()->warn($error);
       $addField = FALSE;
      }

      if($addField){
       //Verify language, application of BVR4
       if (empty($object->additionalParameter['language'])){
         $object->additionalParameter['language'] = "en";
         $this->adms_object->getLogger()->warn("'{$type}' identifies a class that has a '{$type_term}' for which the language tag is missing. It is set automatically to English.");
       }else if (!$this->adms_object->_validator->verifLanguage($object->additionalParameter['language'])) {
         $object->additionalParameter['language'] = "en";
         $this->adms_object->getLogger()->warn("'{$type}' identifies a class that has a '{$type_term}' for which the language tag does not respect the language code syntax specified in RFC 5646. It is set automatically to English.");
       }

       //Checks if the field already exits for that name
       $multiple=FALSE;
       foreach ($field as $key => $value) {
        if ($object->additionalParameter['language'] == $value['value'][$language_field][0]['value']) {
         $addField = FALSE;
         $multiple=TRUE;
        }
       }
       if($multiple){
        $this->adms_object->getLogger()->warn("'{$type}' identifies an element for which a '{$type_term}' property has been used more than once for the same language ('{$object->additionalParameter['language']}'). Multiple values are only allowed as long as they are represented in different languages.");
       }
      }

      if (TRUE == $addField) {
        $field = array(
          'type' => $property['extra_language_type'],
          'value' => array(
            $language_field => array(array('value' => $object->additionalParameter['language'])),
            $language_value => array(array('value' => $object->value))
            )
          );
        if($object->additionalParameter['language']=='en' && $node_field!="" && !isset($this->adms_object->node->{$node_field})){
          $this->adms_object->node->{$node_field} = $value;
        }elseif (0 == count($this->adms_object->node->{$field_name})){
          $this->adms_object->node->{$field_name}[0] = $field;
        }else {
          $this->adms_object->node->{$field_name}[] = $field;
        }
      }
    }
  /**
   * Method for map the property of the value type to the node
   * @var $property
   * @var $object
   * @return bool
   */
  public function importValue($property, $object){
    //if the $object is string, obtain the value directly from the variable

    if(is_string($object)){
      $value = $object;
    } else {
      $value = $object->value;
    }
    $field_name = trim($property['node_field']);
    $field_value_key = trim($property['value_key']);
    if(isset($object->additionalParameter['orig_term'])&&!empty($object->additionalParameter['orig_term'])){
     $type_term = $object->additionalParameter['orig_term'];
    }else{
     $type_term = $property['namespace']['admsx'];
    }

    if(!$field_name) return false;

    // Cleanup unwanted chars in title
    if ($field_name === 'title') {
      $value = preg_replace('/[\\t\\n\\f\\r]/', ' ', $value);
      //$object->value =  preg_replace('/^(.*)[^\w;&].*?$/', '\\1', $value);
    }

    // description field parsing
    if(isset($object->additionalParameter['language'])) {
      //treatement of description

      if (empty($object->additionalParameter['language'])) {
        $object->additionalParameter['language'] = 'en';
        $this->adms_object->getLogger()->warn("The name '{$object->value}' of the {$this->adms_object->node->type} with URI '{$this->adms_object->uri}' " .
          "doesn't have any language attribute. It is set automatically to English.");
      } else if (!$this->adms_object->_validator->verifLanguage($object->additionalParameter['language'])) {
        // force english lang (revision of BVR4: assume english when missing or unknown lang)
        $language = $object->additionalParameter['language'];
        $object->additionalParameter['language'] = 'en';
        $this->adms_object->getLogger()->warn("The name '{$object->value}' of the {$this->adms_object->node->type} with URI '{$this->adms_object->uri}' " .
          "doesn't have an allowed language attribute ('$language'). It is set automatically to English.");
      }

      if ($object->additionalParameter['language'] == "en" && empty($field_value_key)) {
        // only set if not already set (revision of BVR5: keep only the first match)
        if (empty($this->adms_object->node->$field_name)) {
          $this->adms_object->node->$field_name = $object->value;
        } else {
          $this->adms_object->getLogger()->warn("'{$this->adms_object->node->type}' identifies an element for which a '{$type_term}' property has been used more than once for the same language ('{$object->additionalParameter['language']}'). Multiple values are only allowed as long as they are represented in different languages.");
        }
      } else {
        //JIRA 669 , application of BVR4
        //all text must have an language
        if ($object->additionalParameter['language'] != '' && $this->adms_object->_validator->verifLanguage($object->additionalParameter['language'])) {
		  if(isset($property['extra_language'][0])&&!empty($property['extra_language'][0])){
          $field_name = $property['extra_language'][0];
          $tab_description[] = array(
            'type' => 'language_textarea',
            'value' => array(
              'field_language_textfield_lang' => array(array('value' => $object->additionalParameter['language'])),
              'field_language_textarea_name' => array(array('value' => $object->value)))
            );
			}
        }
      }
    }else{
      $field_extra_name = $property['extra_field'];
      if(isset($field_extra_name)){
        if (empty($this->adms_object->node->$field_name)) {
          $this->adms_object->node->$field_name = $object->value;
        } else {
          $this->adms_object->node->$field_extra_name = $object->value;
        }
      }
    }
    //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1168
    //Description must have at least one word
    if(($this->adms_object->node->type=='asset_release' || $this->adms_object->node->type=='repository')
    && $field_name=='body' && str_word_count($value)<1){
	 $error = "{$this->adms_object->id} identifies a {$type_term} that has this description '{$value}'. A description should contain at least 1 words. Please update the description and load the data again";
	 $error = preg_replace('/(^| )a ([aeiouAEIOU])/', '$1an $2', $error);
     $this->adms_object->getLogger()->warn($error);
     return false;
    }else{
     if($field_value_key!=''){
       if(isset($tab_description)) {
         $this->adms_object->node->{$field_name}[] = $tab_description;
       }else{
         $this->adms_object->node->{$field_name}[] = array($field_value_key => $value);
       }
     }else{
       $this->adms_object->node->{$field_name} = $value;
     }
    }
  }
}
