<?php
/**
 * Created by PhpStorm.
 * User: antartem
 * Date: 26/11/13
 * Time: 15:46
 */

namespace Common;


class Validator {

  /**
   * Reference to the ADMS Object
   * @var ADMSObject
   */
  public $adms_object;


  /**
   * Check whether the given lang code is not empty and exists in the list of allowed languages
   * @$lang language code
   * @access private
   * return bool
   */
  public function verifLanguage($lang) {
    return isset($this->languages[$lang]);
  }


  function __construct(ADMSObject &$adms_object) {
    $this->adms_object = &$adms_object;
    $this->languages = locale_language_list();
  }

  /**
   * This method check if an URI is in the correct format
   * @param $object the path to the file which should be loaded
   * @return true or false
   * @access private
   */
  public function verifURI($uri) {
    return _isa_toolbox_valid_uri($uri);
  }

  /**
   * This method check if an URL is in the correct format and check that the URL is reachable
   * @param $object the path to the file which should be loaded
   * @return true or false
   * @access private
   */
  public function verifURL($url) {
    if (!empty($url) && !link_validate_url($url)) {
      return false;
    }
    //don't return a 4xx or 5xx error
    $url_check = drupal_http_request($url);
    return ($url_check->code < 400 || $url_check->code >= 600);
  }

  /**
   * Check property cardinality
   * @access public
   */
  public function checkCardinality() {
    $type = $this->adms_object->getNodeProperty('type');
    foreach($this->adms_object->getProperties() as $name => $property){
      $min = $property['cardinality'][0];
      $max = $property['cardinality'][1];
      $object = $this->adms_object->$name;
      if(isset($object->additionalParameter['orig_term'])&&!empty($object->additionalParameter['orig_term'])){
       $type_term = $object->additionalParameter['orig_term'];
      }else{
       $type_term = $property['namespace']['admsx'];
      }
      if($min >= 1 && (count($this->adms_object->getNodeProperty($property) )< $min )) {
       if($name=='publisher' && $type=='asset_release'){
        $this->adms_object->tStatus = 'error';
        $message = "{$this->adms_object->uri} identifies a {$type} for which no {$type_term} has been defined. Without this mandatory attribute, it cannot be imported in the federation. Please provide at least one {$property['namespace']['admsx']} and load the data again.";
        $message = preg_replace('/(^| )a ([aeiouAEIOU])/', '$1an $2', $message);
		    $this->adms_object->error_message = $message;
        $this->adms_object->getLogger()->warn($message);
        return false;
       }else{
	      $message = "'{$this->adms_object->uri} identifies a '{$type}' for which no {$type_term} has been defined. Every {$type} in the federation must have at least 1 {$property['namespace']['admsx']}.";
	      $message = preg_replace('/(^| )a ([aeiouAEIOU])/', '$1an $2', $message);
          $this->adms_object->getLogger()->warn($message);
       }
      }
      if($max >= 1 && (count($this->adms_object->getNodeProperty($property)) > $max )) {
        $this->adms_object->getLogger()->warn("{$type} has more than $max {$type_term} attribute(s) associated. The system will save only the first one.");
      }
    }
  }

  /**
   * This function check if a node is blacklisted
   * @param $nid the node to load
   * @return true if blacklisted or false
   * @access private
   */
  public function checkBlacklist() {
    if(isset($this->node->nid)){
      // No need to load the full node, just create a stdClass and set the "nid"
      $node = new \stdClass();
      $node->nid = $this->node->nid;
      //check if blacklisted
      $sid_workflow = workflow_node_current_state($node);

      $result = $sid_workflow == ISA_SID_ASSET_BLACKLISTED;
      if(!$result) {
        $this->adms_object->tStatus = 'error';
        $message = "This asset release with URI '{$this->adms_object->uri}' is blacklisted.";
        $this->adms_object->error_message = $message;
        $this->adms_object->getLogger()->warn($message);
      }
    }
  }


  /**
   * This function checks uri
   * @param $uri
   * @return true if uri is valid or false
   * @access public
   */
  public function checkURI() {
    $uri = $this->adms_object->uri;
    if(!isset($this->adms_object->output['repository_uri'])){
      $repository_nid = $this->adms_object->output['repository_id'];
      $repository = node_load($repository_nid);
      $this->adms_object->output['repository_uri'] = $repository->field_id_uri[0]['value'];
    }

    switch ($this->adms_object->item_class) {
      case 'ADMSx\RdfClasses\AssetRepository':
        //$this->adms_object->getLogger()->debug("Current URI '{$uri}', Repository URI '{$this->adms_object->output['repository_uri']}'");
        //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-736
        //Trim repository url to do the comparison
        if (isset($this->adms_object->output['repository_uri']) && $uri != trim($this->adms_object->output['repository_uri'], ISA_URI_TRIM_CHARS)) {
          $this->adms_object->getLogger()->warn("The repository with URI '{$uri}' has been ignored. Only the repository with URI '{$this->adms_object->output['repository_uri']}' is treated.");
        }

        break;
      case 'ADMSx\RdfClasses\Asset':
        //$this->adms_object->getLogger()->debug($this->repositoryOrigin[0]->value . " > " . $this->adms_object->output['repository_uri']);
        //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-736
        //Trim repository url to do the comparison
        if(isset($this->adms_object->isPartOf[0])){
          if (trim($this->adms_object->isPartOf[0]->value, ISA_URI_TRIM_CHARS) != trim($this->adms_object->output['repository_uri'], ISA_URI_TRIM_CHARS)) {
            $this->adms_object->getLogger()->warn("The asset with URI '{$uri}' is not included in the current repository. It is ignored");
          }
        }

        break;
    }

    //check if uri is conform
    if (!$this->verifURI($uri)) {
      $message = "The URI value '{$uri}' of '{$this->adms_object->item_class}' does not respect the URI syntax specified in  RFC 3986 [http://www.ietf.org/rfc/rfc3986.txt]. Please correct this and load the data again";
      $this->adms_object->getLogger()->info($message);
      //put the status of the item to error
      $this->adms_object->tStatus = 'error';
      $this->adms_object->error_message = $message;
    }
  }

  /**
   * Check if node body is correct
   * @return bool
   */
  public function checkBodyField(){
    $type = $this->adms_object->getNodeProperty('type');
    $body = $this->adms_object->getNodeProperty('body');
    if ($type == 'repository' || $type == 'asset_release') {
      if (empty($body)) {
        // Put the status of the item to error
        $this->adms_object->tStatus = 'error';
        $message = "'{$this->adms_object->uri}' identifies a {$type} that does not have a description (dcterms:description). Please provide a description and load the data again.";
        $message = preg_replace('/(^| )a ([aeiouAEIOU])/', '$1an $2', $message);
		$this->adms_object->error_message = $message;
        $this->adms_object->getLogger()->warn($message);
      }
    }
  }

  /**
   * Check if node title is correct
   * the name is mandatory for all the contenttype
   * (excluded item types: contact_point, identifier)
   * @return bool
   */
  public function checkTitleField(){
    $type = $this->adms_object->getNodeProperty('type');
    $title = $this->adms_object->getNodeProperty('title');
    if ($type != 'identifier' && $type!='software_identifier') {
      if (empty($title)) {
        //put the status of the item to error
        $this->adms_object->tStatus = 'error';
        $message = "'{$this->adms_object->uri}' identifies a {$type} object that is missing a name (rdfs:label). Without this mandatory attribute, it cannot be imported in the federation. Please provide a name and load the data again.";
        $message = preg_replace('/(^| )a ([aeiouAEIOU])/', '$1an $2', $message);
		$this->adms_object->error_message = $message;
        $this->adms_object->getLogger()->warn($message);
      }
    }//end of name treatment
  }

  /**
   * Check if node title is correct
   * the name is mandatory for all the contenttype
   * (excluded item types: contact_point, identifier)
   * @return bool
   */
  public function checkAccessURL(){
    $type = $this->adms_object->getNodeProperty('type');
    // Check if node correct
    if ($type == 'distribution' || $type == 'repository') {
      //put the status of the item to error
      if (count($this->adms_object->getNodeProperty($this->adms_object->properties['accessURL']['node_field'])) == 0){
        $this->adms_object->tStatus = 'error';
        $message = "'{$this->adms_object->uri}' identifies a {$type} which does not have an accessURL (dcat:accessURL). Please provide a description and load the data again.";
        $message = preg_replace('/(^| )a ([aeiouAEIOU])/', '$1an $2', $message);
		$this->adms_object->error_message = $message;
        $this->adms_object->getLogger()->warn($message);
      }
    }
  }
}