<?php
namespace test\AdmsTestFramework;
class AdmsTestFramework {
  /**
   * Test url, used to get the harvest url
   */
  private $domain = "";
  /**
   * Path to generated test
   */
  private $testPath = '';
  /**
   * Folder to save test results
   */
  private $testFolder = '';
  /**
   * File name for the initial rdf file to import
   */
  private $testFile = '';
  /**
   * File for the rdf file exported from initial import
   */
  private $exportFile = '';
  /**
   * Parsed repository data
   */
  private $repository = array();
  /**
   * Imported repository node
   */
  private $importedRepository;
  /**
   * Nid list of the repository structure, used to delete them
   */
  private $importedNodes = array();
  /**
   * Repository structure in json
   */
  private $importedJson = '';
  /**
   * Command line options
   */
  public $switches;
  /**
   * Associated functions to validate command line options
   */
  public $validation;
  /**
   * 
   */
  public function __construct($domain = "http://localhost/", $testPath = 'sites/default/files/', $testFolder = 'admsx_test/') {
    $this->domain = $domain;
    $this->testPath = $testPath;
    $this->testFolder = $testFolder;
    $this->switches = $this->switchesDefinition();
    $this->validation = $this->switchesValidation();
  }
  /**
   * Launch script configuration
   */
  public function configure ($argv){
    foreach ($this->validation as $key => $value) {
      $this->checkArguments($argv,$value['function']);
    }
    for ($i=0; $i < count($argv); $i++) {
      1 === $i % 2 ? $this->switches[$argv[$i]]($argv[$i+1]) : false;
    }
  }
  /**
   * Launch test
   */
  public function launch($delete = true, $compare = true) {
    $this->parseRdf($this->testFile); 
    $this->createNode($this->testFile);
    system('php src_adms/cronjob');
    $this->getExport($this->importedRepository->nid, "original_import_export.rdf");
    $this->getNodes($this->importedRepository, "original_import.json");
    if (true === $delete) {
      $this->deleteNodes($this->importedNodes);
      if (true == $compare) {
        $this->parseRdf($this->exportFile);
        $this->createNode($this->exportFile);
        system('php src_adms/cronjob');
        $this->getExport($this->importedRepository->nid, "imported_export.rdf");
        $this->getNodes($this->importedRepository, "import_import.json");
        $this->deleteNodes($this->importedNodes);
      }
    }
  }
  /**
   * Allows configuration of options with the actions to launch on object construct
   */
  public function switchesDefinition ($switches = array()) {
    if (count($switches) > 0) {
      return $switches;
    } else {
      $that = $this;
      return array(
        "-l" => function($value) {
          $_SERVER['SCRIPT_NAME'] = "/" . $value;
        },
        "-f" => function($value) use ($that){
          $that->rdfToFile($value);
        },
        "-h" => function($value) use ($that){
          $that->rdfToFile($value);
        }
        );
    }
  }
  /**
   * Allows configuration of validation with actions to launch on object construct
   */
  public function switchesValidation($validation = array()){
    if (count($validation) > 0) {
      return $validation;
    }else {
      $switches = array_keys($this->switches);
      return array(
        "min_arguments" => array(
          'function' => function($value) {
            return 1 >= count($value) ? "No arguments passed" : "";
          }
          ),
        "rdf_type" => array(
          'function' => function($value){
            return in_array('-h', $value) && in_array('-f', $value) ? "Set only one rdf source" : "";
          }
          ),
        "value_for_switch" => array(
          'function' => function($value) use ($switches) {
            for ($i=0; $i <= count($value); $i++) {
              if (in_array($value[$i], $switches) and (in_array($value[$i + 1], $switches) or !$value[$i + 1])) {
                return "Set value for switch: " . $value[$i]; 
              }
            } 
            return "";
          }
          ),
        );
    }
  }
  /**
   * Checks command line arguments are valid. Uses the configuration from $validation
   */
  public function checkArguments ($arguments, $callback) {
    if ("" !== $msg = $callback($arguments)) {
      echo "
      $msg
      Help:
      -l set Drupal installation domain
      -f set local rdf file [set -f or -h]
      -h set external url to get rdf file [set -f or -h]
      ";
      exit;
    }
  }
  /**
   * Saves the file configured in the command line in the test path to use it during the test
   */
  public function rdfToFile($rdf){
    if (!is_dir($this->testPath . $this->testFolder)) {
      mkdir($this->testPath . $this->testFolder);
    }
    file_put_contents( $this->testPath . $this->testFolder . "original_import.rdf", file_get_contents($rdf));
    $this->testFile = $this->testPath . $this->testFolder . "original_import.rdf";
  }
  /**
   * Gets the repository data to create the repository node skeleton
   */
  private function parseRdf($file){
    $rdf = file_get_contents($file);
    $rdfObject = new \EasyRdf_Graph();
    $rdfObject->parseFile($file);
    $rdfArray = $rdfObject->toArray();
    $targetValue = 'http://www.w3.org/ns/adms#SemanticAssetRepository';
    $aux = function($val, $array_key, $path) use($targetValue, &$aux) {
      foreach($val as $array_key => $value){
        if(is_array($value)){
          if ($value[0]['value'] == $targetValue) {
            return $path;
          } else {
            $path[] = $value;
            return $aux($val[$array_key],$array_key, $path);
          }  
        }
      }
    };
    $this->repository = array_pop($aux($rdfArray, '', array()));
  }
   /**
   * Creates repository node
   */
   private function createNode($file) {
    global $user;
    $this->setUser('moderator');
    $uid = $user->uid;
    $node = new \stdClass();
    $node->type = 'repository';
    $node->uid = $uid;
    $node->title = $this->repository['http://www.w3.org/2000/01/rdf-schema#label'][0]['value'];
    $node->body = $this->repository['http://purl.org/dc/terms/description'][0]['value'];
    $node->field_id_uri[0]['value'] = $this->repository['http://www.w3.org/ns/adms#accessURL'][0]['value'];
    $node->field_repository_harvestfreq[0]['value'] = 0;
    $node->field_repository_harvestmethod[0]['value'] = 1;
    $node->field_repository_harvesturl[0]['value'] = $this->domain . $file;
    $node->field_repository_url[0]['url'] = $this->repository['http://www.w3.org/ns/adms#accessURL'][0]['value'];
    node_save($node);
    workflow_execute_transition($node,45);
    _repository_harvest($node);
    $this->importedRepository = $node;
    $this->setUser('super_user');
  }
  /**
   * Sets a user to use from the role passed
   */
  private function setUser($rol) {
    global $user;
    switch($rol){
      case 'moderator':
      $sql = "SELECT *
      FROM users 
      INNER JOIN users_roles  ON users.uid = users_roles.uid
      INNER JOIN role ON users_roles.rid = role.rid AND role.name='moderator'
      WHERE users.uid!=1 and users.status=1
      LIMIT 0,1";
      $uid = db_result(db_query($sql,$rol));
      break;
      case 'super_user':
      $uid = 1;
      break;
    }
    $user = user_load($uid);
  }
  /**
   * Gets the rdf export from the repository
   */
  private function getExport($nid, $file) { 
    try {
      $output = '';
        //export of drupal node
      $adapter = \Adapters\AdapterFactory::getAdapter('drupal');
      $data = $adapter->load($nid);
      $adapter = \Adapters\AdapterFactory::getAdapter();
      $adapter->export($data, $output);
      $this->exportFile = $this->testPath . $this->testFolder . $file;
      file_put_contents($this->exportFile, $output);
    } catch (\Exception $e) {
      $logger->fatal($e->getMessage());
      return $e->getMessage();
    }
  }
  /**
   * Returns the fields that are nodereference
   */
  private function nodeReferenceTypes($fields) {
    $nodeReference = array();
    foreach($fields as $key => $value) {
      if ("nodereference" == $value['type']) {
        $nodeReference[] = $key;
      }
    }
    return $nodeReference;
  }
  /**
   * Builds the repository tree and saves it to a json file
   */
  private function getNodes($node, $file) {
    $node->repository_tree = $this->getRepositoryNodes($node->nid,array("adms_software_proyect",'asset_release')); 
    $this->importedJson = $this->jsonFormat(json_encode($node));
    file_put_contents($this->testPath . $this->testFolder . $file, $this->importedJson);
  }
  /**
   * Starts the generation of the repository tree
   */
  private function getRepositoryNodes($nid, $node_types) {
    $this->importedNodes[] = $nid;
    $group_nodes = og_group_child_nids($nid);
    $nodes = array();
    foreach($group_nodes as $key => $value) {
      $in = "('" . implode("','", $node_types) . "')";
      $sql = "SELECT type FROM node WHERE type IN $in AND nid=%d";
      $type = db_result(db_query($sql,$value));
      if ("" != $type) {
        $root_node = node_load($value);
        $this->importedNodes[] = $root_node->nid;
        $node = $this->getReferencedNode($root_node);
        $nodes[$type][] = $node;
      } 
    }
    return $nodes;
  }
  /**
   * Recursive function get and node structure following all node referenced fields
   */
  private function getReferencedNode($node) {
    $fields = content_fields(NULL, $node->type);
    $fields = $this->nodeReferenceTypes($fields);
    foreach($node as $key => $value) {
      if (in_array($key,$fields)) {
        foreach($value as $sub_key => $sub_value) {
          $node->{$key}[$sub_key]['node'] = $this->getReferencedNode(node_load($value[0]['nid']));
          $this->importedNodes[] = $value[0]['nid'];
        }
      }
    }
    return $node;
  }
  /**
   * Deletes nodes
   */
  private function deleteNodes($nodes) {
    foreach($nodes as $key => $value) {
      node_delete($value);
    }
  }
  /**
  * Format a flat JSON string to make it more human-readable
  *
  * @param string $json The original JSON string to process
  * When the input is not a string it is assumed the input is RAW
  * and should be converted to JSON first of all.
  * @return string Indented version of the original JSON string
  * original code: http://www.daveperrett.com/articles/2008/03/11/format-json-with-php/
  * adapted to allow native functionality in php version >= 5.4.0
  */
  private function jsonFormat($json) {
    if (!is_string($json)) {
      if (phpversion() && phpversion() >= 5.4) {
        return json_encode($json, JSON_PRETTY_PRINT);
      }
      $json = json_encode($json);
    }
    $result = '';
    $pos = 0; // indentation level
    $strLen = strlen($json);
    $indentStr = "\t";
    $newLine = "\n";
    $prevChar = '';
    $outOfQuotes = true;

    for ($i = 0; $i < $strLen; $i++) {
      // Grab the next character in the string
      $char = substr($json, $i, 1);

      // Are we inside a quoted string?
      if ($char == '"' && $prevChar != '\\') {
        $outOfQuotes = !$outOfQuotes;
      }
      // If this character is the end of an element,
      // output a new line and indent the next line
      else if (($char == '}' || $char == ']') && $outOfQuotes) {
        $result .= $newLine;
        $pos--;
        for ($j = 0; $j < $pos; $j++) {
          $result .= $indentStr;
        }
      }
      // eat all non-essential whitespace in the input as we do our own here and it would only mess up our process
      else if ($outOfQuotes && false !== strpos(" \t\r\n", $char)) {
        continue;
      }

      // Add the character to the result string
      $result .= $char;
      // always add a space after a field colon:
      if ($char == ':' && $outOfQuotes) {
        $result .= ' ';
      }

      // If the last character was the beginning of an element,
      // output a new line and indent the next line
      if (($char == ',' || $char == '{' || $char == '[') && $outOfQuotes) {
        $result .= $newLine;
        if ($char == '{' || $char == '[') {
          $pos++;
        }
        for ($j = 0; $j < $pos; $j++) {
          $result .= $indentStr;
        }
      }
      $prevChar = $char;
    }
    return $result;
  }
}
